
{***************************************************************************}
{                                                                           }
{     Unit: 'SpeechRecogPaintDiagsFile.pas'                                 }
{     ( containing class defining variables and methods for diagram )       }
{                                                                           }
{     - belonging to:                                                       }
{       Form:    'SpeechRecogForm.dfm'                                      }
{       Unit:    'SpeechRecogFormFile.pas'                                  }
{       Project: 'SpeechRecognition.dpr'                                    }
{       DLL:     'SpeechRecognition.dll'                                    }
{       Program: 'ResNeuronum.exe'                                          }
{       ( simulating Neural Networks )                                      }
{                                                                           }
{     - written in Borland Delphi 2.0                                       }
{                                                                           }
{     Copyright (c) 1996-97 TU Ilmenau/ Germany                             }
{                                                                           }
{     Written by:                                                           }
{       Christian Heller                                                    }
{       Grenzgraben 04                                                      }
{       D-98714 Sttzerbach                                                 }
{       Thringen                                                           }
{       Deutschland/ Germany/ Allemagne                                     }
{       Tel.:   +49/ (0)36784/ 50471 (Euro-ISDN)                            }
{       or Tel: +49/ (0)36784/ 50690                                        }
{       Fax G3: +49/ (0)36784/ 50992 (Euro-ISDN)                            }
{       e-mail: Christian.Heller@rz.tu-ilmenau.de                           }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/christian/enmain.htm     }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/resneuronum/enmain.htm   }
{                                                                           }
{     ( all variables & methods in alphabetical order                       }
{     or as appearing in the unit header, respectively )                    }
{                                                                           }
{***************************************************************************}

unit SpeechRecogPaintDiagsFile;

interface

uses
  Classes,
  SysUtils,
  SpeechRecogDynArrayFile;

type
  { Class 'TPaintDiags' containing necessary data to paint diagrams }
  TPaintDiags = class(TObject)
  private
    FBottom: Integer;
    FDiagType: Integer;
    FFactX: Double;
    FFactY: Double;
    FLeft: Integer;
    FMaxX: Double;
    FMaxY: Double;
    FMidHoriz: Integer;
    FMidVert: Integer;
    FMinX: Double;
    FMinY: Double;
    FRight: Integer;
    FTop: Integer;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure SetDiagData(const DataArray: TDoubleArray);
    property Bottom: Integer read FBottom write FBottom;
    property DiagType: Integer read FDiagType write FDiagType;
    property FactX: Double read FFactX write FFactX;
    property FactY: Double read FFactY write FFactY;
    property Left: Integer read FLeft write FLeft;
    property MaxX: Double read FMaxX write FMaxX;
    property MaxY: Double read FMaxY write FMaxY;
    property MidHoriz: Integer read FMidHoriz write FMidHoriz;
    property MidVert: Integer read FMidVert write FMidVert;
    property MinX: Double read FMinX write FMinX;
    property MinY: Double read FMinY write FMinY;
    property Right: Integer read FRight write FRight;
    property Top: Integer read FTop write FTop;
  end;

implementation

constructor TPaintDiags.Create;
begin
  inherited Create;
end;

destructor TPaintDiags.Destroy;
begin
  inherited Destroy;
end;

procedure TPaintDiags.SetDiagData(const DataArray: TDoubleArray);

  { Logarithmise Value to Base }
  function Logarithmise(const Base: Double; const Value: Double): Double;
  begin
    try
      Result := (ln(Value) / ln(Base));
    except
      on Exception do Result := 0;
    end;
  end;

var
  i: Integer;
  RangeX, RangeY: Double;
begin
  if ((DataArray <> nil) and (DataArray.Count > 0)) then begin
    { Get minimum and maximum of data }
    FMinX := 0;
    FMaxX := DataArray.Count;
    FMinY := DataArray[0];
    FMaxY := DataArray[0];
    for i := 0 to (DataArray.Count - 1) do begin
      if (DataArray[i] < FMinY) then begin
        FMinY := DataArray[i];
      end;
      if (DataArray[i] > FMaxY) then begin
        FMaxY := DataArray[i];
      end;
    end;
    { Calculate ranges for x- and y-axis }
    RangeX := Trunc(FMaxX - FMinX);
    try
      FFactX := (Abs((FRight - 5) - FLeft) / RangeX);
    except
      on Exception do FFactX := 0;
    end;
    RangeY := 1;
    if ((FDiagType = 0) or (FDiagType = 1) or (FDiagType = 2)) then RangeY := (32767 - (-32768));
    if (FDiagType = 3) then RangeY := Trunc((20 * Logarithmise(10, FMaxY)) - 0);
    if (FDiagType = 4) then begin
      if (Abs(FMinY) > Abs(FMaxY)) then FMaxY := Abs(FMinY) else FMaxY := Abs(FMaxY);
      RangeY := Trunc((2 * FMaxY) - 0);
    end;
    if ((FDiagType = 5) or (FDiagType = 6)) then RangeY := Round(FMaxY - 0);
    try
      FFactY := (Abs(FBottom - (FTop + 5)) / RangeY);
    except
      on Exception do FFactY := 0;
    end;
  end
  else begin
    FFactX := 0;
    FFactY := 0;
    FMaxX := 0;
    FMaxY := 0;
    FMinX := 0;
    FMinY := 0;
  end;
end;

end.
