
{***************************************************************************}
{                                                                           }
{     Form: 'BackPropForm.dfm'                                              }
{     Unit: 'BackPropFormFile.pas'                                          }
{     ( containing Neural Net 'BackPropagation' )                           }
{                                                                           }
{     - belonging to:                                                       }
{       Project: 'BackPropagation.dpr'                                      }
{       DLL:     'BackPropagation.dll'                                      }
{       Program: 'ResNeuronum.exe'                                          }
{       ( simulating Neural Networks )                                      }
{                                                                           }
{     - written in Borland Delphi 2.0                                       }
{                                                                           }
{     Copyright (c) 1996-97 TU Ilmenau/ Germany                             }
{                                                                           }
{     Written by:                                                           }
{       Christian Heller                                                    }
{       Grenzgraben 04                                                      }
{       D-98714 Sttzerbach                                                 }
{       Thringen                                                           }
{       Deutschland/ Germany/ Allemagne                                     }
{       Tel.:   +49/ (0)36784/ 50471 (Euro-ISDN)                            }
{       or Tel: +49/ (0)36784/ 50690                                        }
{       Fax G3: +49/ (0)36784/ 50992 (Euro-ISDN)                            }
{       e-mail: Christian.Heller@rz.tu-ilmenau.de                           }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/christian/enmain.htm     }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/resneuronum/enmain.htm   }
{                                                                           }
{     ( all variables & methods in alphabetical order                       }
{     or as appearing in the unit header, respectively )                    }
{                                                                           }
{***************************************************************************}

unit BackPropFormFile;

interface

uses
  { Net component created especially for the project }
  BackPropNet,
  Buttons,
  Classes,
  ComCtrls,
  Controls,
  Dialogs,
  ExtCtrls,
  FileDlg,
  Forms,
  Graphics,
  Grids,
  Messages,
  Printers,
  Spin,
  StdCtrls,
  SysUtils,
  Windows;

type
  { Record storing info about kind of data processing }
  TEvalProc = record
    ProcType: Integer;
    CorrWith: Integer;
    CorrDepth: Integer;
    CorrName: string;
    OperWith: Integer;
    OperKind: Integer;
    OperName: string;
    OperAdaptConst: Double;
    FiltConst: Double;
    FiltName: string;
    StatMoment: Integer;
    StatMomentName: string;
    StatAdaptConst: Double;
  end;

  { Record for storing information about curves on page 'Evaluation' }
  TEvalCurve = record
    CurrData: Integer;
    Cycle: Integer;
    Proc: TEvalProc;
    Layer: Integer;
    Neuron: Integer;
    Weight: Integer;
  end;

  { Definition of form }
  TBackPropForm = class(TForm)
    { Section 'Form' }
    BackPropBackPropNet: TBackPropNet;
    BackPropLabelAuthor: TLabel;
    BackPropNetFileDialog: TNetFileDialog;
    BackPropPageControl: TPageControl;
    BackPropPrintDialog: TPrintDialog;
    BackPropStatusBar: TStatusBar;
    { TabSheet 'Configuration' }
    ConfigBitBtnNewInit: TBitBtn;
    ConfigBitBtnOpen: TBitBtn;
    ConfigBitBtnSave: TBitBtn;
    ConfigComboBoxFunctAct: TComboBox;
    ConfigComboBoxFunctOutp: TComboBox;
    ConfigComboBoxRuleProp: TComboBox;
    ConfigEditFileCfg: TEdit;
    ConfigEditFunctActFactScal: TEdit;
    ConfigEditFunctOutpMax: TEdit;
    ConfigEditFunctOutpMin: TEdit;
    ConfigEditFunctOutpSlope: TEdit;
    ConfigEditFunctOutpThresh: TEdit;
    ConfigGroupBoxFileCfg: TGroupBox;
    ConfigGroupBoxFunctAct: TGroupBox;
    ConfigGroupBoxFunctOutp: TGroupBox;
    ConfigGroupBoxLay: TGroupBox;
    ConfigGroupBoxRuleProp: TGroupBox;
    ConfigLabelConvention: TLabel;
    ConfigLabelFileCfg: TLabel;
    ConfigLabelFunctActFactScal: TLabel;
    ConfigLabelFunctActFactScalSymb: TLabel;
    ConfigLabelFunctActSymb: TLabel;
    ConfigLabelFunctOutpMax: TLabel;
    ConfigLabelFunctOutpMaxSymb: TLabel;
    ConfigLabelFunctOutpMin: TLabel;
    ConfigLabelFunctOutpMinSymb: TLabel;
    ConfigLabelFunctOutpSlope: TLabel;
    ConfigLabelFunctOutpSlopeSymb: TLabel;
    ConfigLabelFunctOutpSymb: TLabel;
    ConfigLabelFunctOutpThresh: TLabel;
    ConfigLabelFunctOutpThreshSymb: TLabel;
    ConfigLabelNrLay: TLabel;
    ConfigLabelNrNeur: TLabel;
    ConfigLabelRulePropSymb: TLabel;
    ConfigListBoxLay: TListBox;
    ConfigListBoxNrNeur: TListBox;
    ConfigSpinEditNrLay: TSpinEdit;
    ConfigSpinEditNrNeur: TSpinEdit;
    ConfigTabSheet: TTabSheet;
    { TabSheet 'Evaluation' }
    EvalBitBtnNewInit: TBitBtn;
    EvalBitBtnPrint: TBitBtn;
    EvalBitBtnPrintAll: TBitBtn;
    EvalCurveLabel1: TLabel;
    EvalCurveLabel2: TLabel;
    EvalCurveLabel3: TLabel;
    EvalCurveLabel4: TLabel;
    EvalCurveSpeedButton1: TSpeedButton;
    EvalCurveSpeedButton2: TSpeedButton;
    EvalCurveSpeedButton3: TSpeedButton;
    EvalCurveSpeedButton4: TSpeedButton;
    EvalGroupBoxCurve: TGroupBox;
    EvalGroupBoxDataProc: TGroupBox;
    EvalGroupBoxElement: TGroupBox;
    EvalGroupBoxSingleValue: TGroupBox;
    EvalLabelCycle: TLabel;
    EvalLabelLayer: TLabel;
    EvalLabelNeuron: TLabel;
    EvalLabelNeuronSymb: TLabel;
    EvalLabelWeight: TLabel;
    EvalLabelWeightSymb: TLabel;
    EvalPanelImageSingleValue: TImage;
    EvalPanelSingleValue: TPanel;
    EvalProcSpeedButton0: TSpeedButton;
    EvalProcSpeedButton1: TSpeedButton;
    EvalProcSpeedButton2: TSpeedButton;
    EvalProcSpeedButton3: TSpeedButton;
    EvalProcSpeedButton4: TSpeedButton;
    EvalProcSpeedButton5: TSpeedButton;
    EvalProcSpeedButton6: TSpeedButton;
    EvalProcSpeedButton7: TSpeedButton;
    EvalProcSpeedButton8: TSpeedButton;
    EvalRadioGroupData: TRadioGroup;
    EvalSpeedButtonShowDiag: TSpeedButton;
    EvalSpinEditCycle: TSpinEdit;
    EvalSpinEditLayer: TSpinEdit;
    EvalSpinEditNeuron: TSpinEdit;
    EvalSpinEditWeight: TSpinEdit;
    EvalTabSheet: TTabSheet;
    { TabSheet 'Learn' }
    LearnBevelWeightChoice1: TBevel;
    LearnBevelWeightChoice2: TBevel;
    LearnBitBtnBreak: TBitBtn;
    LearnBitBtnNewInit: TBitBtn;
    LearnBitBtnOpen: TBitBtn;
    LearnBitBtnReset: TBitBtn;
    LearnBitBtnStart: TBitBtn;
    LearnBitBtnWeightChoice: TBitBtn;
    LearnCheckBoxAdaptiveLearning: TCheckBox;
    LearnCheckBoxCollectResults: TCheckBox;
    LearnCheckBoxLearnRateRecursive: TCheckBox;
    LearnCheckBoxLearnSlope: TCheckBox;
    LearnCheckBoxLearnThresh: TCheckBox;
    LearnCheckBoxLearnWeights: TCheckBox;
    LearnCheckBoxSave: TCheckBox;
    LearnComboBoxCollectResults: TComboBox;
    LearnEditAdaptConst: TEdit;
    LearnEditAdaptConstFactor: TEdit;
    LearnEditAdaptConstStep: TEdit;
    LearnEditCycleLimit: TEdit;
    LearnEditErrorLimit: TEdit;
    LearnEditFileLrn: TEdit;
    LearnEditLearnRate: TEdit;
    LearnEditLearnRateDec: TEdit;
    LearnEditLearnRateInc: TEdit;
    LearnEditMomentum: TEdit;
    LearnEditParamsMax: TEdit;
    LearnEditParamsMin: TEdit;
    LearnGroupBoxAdaptiveLearning: TGroupBox;
    LearnGroupBoxCollectResults: TGroupBox;
    LearnGroupBoxFileLrn: TGroupBox;
    LearnGroupBoxLearnParams: TGroupBox;
    LearnGroupBoxLearnParamsInit: TGroupBox;
    LearnGroupBoxLearnRateRecursive: TGroupBox;
    LearnGroupBoxLimit: TGroupBox;
    LearnGroupBoxLearnProcess: TGroupBox;
    LearnGroupBoxPresent: TGroupBox;
    LearnGroupBoxWeightChoice: TGroupBox;
    LearnLabelAdaptConst: TLabel;
    LearnLabelAdaptConstFactor: TLabel;
    LearnLabelAdaptConstFactorSymb: TLabel;
    LearnLabelAdaptConstReciprocalSymb: TLabel;
    LearnLabelAdaptConstStep: TLabel;
    LearnLabelAdaptConstStepSymb: TLabel;
    LearnLabelAdaptConstSymb: TLabel;
    LearnLabelCycleLimit: TLabel;
    LearnLabelErrorLimit: TLabel;
    LearnLabelFileLrn: TLabel;
    LearnLabelLearnRate: TLabel;
    LearnLabelLearnRateDec: TLabel;
    LearnLabelLearnRateInc: TLabel;
    LearnLabelLearnRateSymb: TLabel;
    LearnLabelLearnSlope: TLabel;
    LearnLabelLearnThresh: TLabel;
    LearnLabelLearnWeights: TLabel;
    LearnLabelMax: TLabel;
    LearnLabelMin: TLabel;
    LearnLabelMomentum: TLabel;
    LearnLabelMomentumSymb: TLabel;
    LearnLabelWeightChoice1: TLabel;
    LearnLabelWeightChoice2: TLabel;
    LearnLabelWeightChoiceLayer: TLabel;
    LearnLabelWeightChoiceNeuron: TLabel;
    LearnPaintBoxBar: TPaintBox;
    LearnPaintBoxDiag: TPaintBox;
    LearnPaintBoxHinton: TPaintBox;
    LearnPanelAdaptConstReciprocal: TPanel;
    LearnPanelGlobalError: TPanel;
    LearnPanelWeights: TPanel;
    LearnRadioGroupMethod: TRadioGroup;
    LearnSpeedButtonBar: TSpeedButton;
    LearnSpeedButtonDiag: TSpeedButton;
    LearnSpeedButtonHinton: TSpeedButton;
    LearnSpinEditWeightChoice1: TSpinEdit;
    LearnSpinEditWeightChoice2: TSpinEdit;
    LearnSpinEditWeightChoiceLayer: TSpinEdit;
    LearnSpinEditWeightChoiceNeuron: TSpinEdit;
    LearnTabSheet: TTabSheet;
    { TabSheet 'Pattern' }
    PattBevelHorizTable: TBevel;
    PattBevelPatternSet: TBevel;
    PattBevelVertTable: TBevel;
    PattBitBtnDelPatt: TBitBtn;
    PattBitBtnNewPatt: TBitBtn;
    PattBitBtnNewInit: TBitBtn;
    PattBitBtnOpenPatternSet: TBitBtn;
    PattBitBtnPatternSetDimInpDec: TBitBtn;
    PattBitBtnPatternSetDimInpInc: TBitBtn;
    PattBitBtnPatternSetDimTargDec: TBitBtn;
    PattBitBtnPatternSetDimTargInc: TBitBtn;
    PattBitBtnSavePatternSet: TBitBtn;
    PattComboBoxPattCurr: TComboBox;
    PattEditFilePtn: TEdit;
    PattEditPatternSetDimInp: TEdit;
    PattEditPatternSetDimTarg: TEdit;
    PattEditValInpCurr: TEdit;
    PattEditValTargCurr: TEdit;
    PattGroupBoxFilePtn: TGroupBox;
    PattGroupBoxPatt: TGroupBox;
    PattGroupBoxPattSet: TGroupBox;
    PattLabelFilePtn: TLabel;
    PattLabelListInp: TLabel;
    PattLabelListTarg: TLabel;
    PattLabelNrPatt: TLabel;
    PattLabelPattCurr: TLabel;
    PattLabelPatternSetDim: TLabel;
    PattLabelPatternSetDimInp: TLabel;
    PattLabelPatternSetDimTarg: TLabel;
    PattListBoxInp: TListBox;
    PattListBoxTarg: TListBox;
    PattPanelNrPatt: TPanel;
    PattTabSheet: TTabSheet;
    { TabSheet 'Pruning' }
    PrunBitBtnStart: TBitBtn;
    PrunBitBtnBreak: TBitBtn;
    PrunBitBtnMinConfig: TBitBtn;
    PrunBitBtnMinCycles: TBitBtn;
    PrunBitBtnPrimary: TBitBtn;
    PrunGroupBoxConfig: TGroupBox;
    PrunGroupBoxLimit: TGroupBox;
    PrunGroupBoxNeurNo: TGroupBox;
    PrunLabelLimit: TLabel;
    PrunSpinEditLimit: TSpinEdit;
    PrunStringGridNeuronNo: TStringGrid;
    PrunTabSheet: TTabSheet;
    { TabSheet 'Recall' }
    RecallBevelHorizTable: TBevel;
    RecallBevelRecallSet: TBevel;
    RecallBevelVertTable: TBevel;
    RecallBitBtnBreak: TBitBtn;
    RecallBitBtnDelRecallVect: TBitBtn;
    RecallBitBtnNewInit: TBitBtn;
    RecallBitBtnNewRecallVect: TBitBtn;
    RecallBitBtnOpenRecallSet: TBitBtn;
    RecallBitBtnRecallSetDimInpInc: TBitBtn;
    RecallBitBtnRecallSetDimInpDec: TBitBtn;
    RecallBitBtnSaveRecallSet: TBitBtn;
    RecallBitBtnStart: TBitBtn;
    RecallComboBoxRecallVectInp: TComboBox;
    RecallComboBoxRecallVectOutp: TComboBox;
    RecallEditFileRcl: TEdit;
    RecallEditRecallSetDimInp: TEdit;
    RecallEditValInpCurr: TEdit;
    RecallGroupBoxFileRcl: TGroupBox;
    RecallGroupBoxRecallVect: TGroupBox;
    RecallGroupBoxRecallSet: TGroupBox;
    RecallLabelFileRcl: TLabel;
    RecallLabelListInp: TLabel;
    RecallLabelListOutp: TLabel;
    RecallLabelNrRecallVect: TLabel;
    RecallLabelRecallSetDim: TLabel;
    RecallLabelRecallSetDimInp: TLabel;
    RecallLabelRecallSetDimOutp: TLabel;
    RecallLabelRecallVectInp: TLabel;
    RecallLabelRecallVectOutp: TLabel;
    RecallListBoxInp: TListBox;
    RecallListBoxOutp: TListBox;
    RecallPanelNrRecallVect: TPanel;
    RecallPanelRecallSetDimOutp: TPanel;
    RecallTabSheet: TTabSheet;
    { Section 'BackPropNet' }
    procedure BackPropBackPropNetLearnExit(Sender: TBackPropNet;
      const ExitCode: TExitCode);
    procedure BackPropBackPropNetLearnInterval(Sender: TBackPropNet);
    procedure BackPropBackPropNetRecallExit(Sender: TBackPropNet;
      const ExitCode: TExitCode);
    { Section 'BackPropagation' }
    procedure BackPropPageControlChange(Sender: TObject);
    procedure BackPropStatusBarDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    { TabSheet 'Configuration' }
    procedure ConfigBitBtnNewInitClick(Sender: TObject);
    procedure ConfigBitBtnOpenClick(Sender: TObject);
    procedure ConfigBitBtnSaveClick(Sender: TObject);
    procedure ConfigComboBoxFunctActChange(Sender: TObject);
    procedure ConfigComboBoxFunctOutpChange(Sender: TObject);
    procedure ConfigComboBoxRulePropChange(Sender: TObject);
    procedure ConfigEditFunctActFactScalExit(Sender: TObject);
    procedure ConfigEditFunctActFactScalKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure ConfigEditFunctOutpSlopeExit(Sender: TObject);
    procedure ConfigEditFunctOutpMaxExit(Sender: TObject);
    procedure ConfigEditFunctOutpMaxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ConfigEditFunctOutpMinExit(Sender: TObject);
    procedure ConfigEditFunctOutpMinKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ConfigEditFunctOutpSlopeKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure ConfigEditFunctOutpThreshExit(Sender: TObject);
    procedure ConfigEditFunctOutpThreshKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure ConfigListBoxLayClick(Sender: TObject);
    procedure ConfigSpinEditNrLayChange(Sender: TObject);
    procedure ConfigSpinEditNrNeurExit(Sender: TObject);
    procedure ConfigSpinEditNrNeurKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    { TabSheet 'Evaluation' }
    procedure EvalBitBtnNewInitClick(Sender: TObject);
    procedure EvalBitBtnPrintClick(Sender: TObject);
    procedure EvalBitBtnPrintAllClick(Sender: TObject);
    procedure EvalCurveSpeedButtonClick(Sender: TObject);
    procedure EvalProcSpeedButtonClick(Sender: TObject);
    procedure EvalRadioGroupDataClick(Sender: TObject);
    procedure EvalSpeedButtonShowDiagClick(Sender: TObject);
    procedure EvalSpinEditCycleChange(Sender: TObject);
    procedure EvalSpinEditLayerChange(Sender: TObject);
    procedure EvalSpinEditNeuronChange(Sender: TObject);
    procedure EvalSpinEditWeightChange(Sender: TObject);
    { Section 'Form' }
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    { TabSheet 'Learn' }
    procedure LearnBitBtnBreakClick(Sender: TObject);
    procedure LearnBitBtnNewInitClick(Sender: TObject);
    procedure LearnBitBtnOpenClick(Sender: TObject);
    procedure LearnBitBtnResetClick(Sender: TObject);
    procedure LearnBitBtnStartClick(Sender: TObject);
    procedure LearnBitBtnWeightChoiceClick(Sender: TObject);
    procedure LearnCheckBoxAdaptiveLearningClick(Sender: TObject);
    procedure LearnCheckBoxCollectResultsClick(Sender: TObject);
    procedure LearnCheckBoxLearnRateRecursiveClick(Sender: TObject);
    procedure LearnCheckBoxLearnSlopeClick(Sender: TObject);
    procedure LearnCheckBoxLearnThreshClick(Sender: TObject);
    procedure LearnCheckBoxLearnWeightsClick(Sender: TObject);
    procedure LearnEditAdaptConstExit(Sender: TObject);
    procedure LearnEditAdaptConstFactorExit(Sender: TObject);
    procedure LearnEditAdaptConstFactorKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure LearnEditAdaptConstKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditAdaptConstStepExit(Sender: TObject);
    procedure LearnEditAdaptConstStepKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure LearnEditCycleLimitExit(Sender: TObject);
    procedure LearnEditCycleLimitKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditErrorLimitExit(Sender: TObject);
    procedure LearnEditErrorLimitKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditLearnRateDecExit(Sender: TObject);
    procedure LearnEditLearnRateDecKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditLearnRateExit(Sender: TObject);
    procedure LearnEditLearnRateIncExit(Sender: TObject);
    procedure LearnEditLearnRateIncKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditLearnRateKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditMomentumExit(Sender: TObject);
    procedure LearnEditMomentumKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditParamsMaxExit(Sender: TObject);
    procedure LearnEditParamsMaxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnEditParamsMinExit(Sender: TObject);
    procedure LearnEditParamsMinKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LearnPaintBoxBarPaint(Sender: TObject);
    procedure LearnPaintBoxDiagPaint(Sender: TObject);
    procedure LearnPaintBoxHintonPaint(Sender: TObject);
    procedure LearnRadioGroupMethodClick(Sender: TObject);
    procedure LearnSpeedButtonBarClick(Sender: TObject);
    procedure LearnSpeedButtonDiagClick(Sender: TObject);
    procedure LearnSpeedButtonHintonClick(Sender: TObject);
    procedure LearnSpinEditWeightChoice1Change(Sender: TObject);
    procedure LearnSpinEditWeightChoice2Change(Sender: TObject);
    procedure LearnSpinEditWeightChoiceLayerChange(Sender: TObject);
    procedure LearnSpinEditWeightChoiceNeuronChange(Sender: TObject);
    { TabSheet 'Pattern' }
    procedure PattBitBtnDelPattClick(Sender: TObject);
    procedure PattBitBtnNewPattClick(Sender: TObject);
    procedure PattBitBtnNewInitClick(Sender: TObject);
    procedure PattBitBtnOpenPatternSetClick(Sender: TObject);
    procedure PattBitBtnPatternSetDimInpDecClick(Sender: TObject);
    procedure PattBitBtnPatternSetDimInpIncClick(Sender: TObject);
    procedure PattBitBtnPatternSetDimTargDecClick(Sender: TObject);
    procedure PattBitBtnPatternSetDimTargIncClick(Sender: TObject);
    procedure PattBitBtnSavePatternSetClick(Sender: TObject);
    procedure PattComboBoxPattCurrChange(Sender: TObject);
    procedure PattEditPatternSetDimInpExit(Sender: TObject);
    procedure PattEditPatternSetDimInpKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure PattEditPatternSetDimTargExit(Sender: TObject);
    procedure PattEditPatternSetDimTargKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure PattEditValInpCurrExit(Sender: TObject);
    procedure PattEditValInpCurrKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PattEditValTargCurrExit(Sender: TObject);
    procedure PattEditValTargCurrKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PattListBoxInpExit(Sender: TObject);
    procedure PattListBoxInpKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PattListBoxTargExit(Sender: TObject);
    procedure PattListBoxTargKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    { TabSheet 'Pruning' }
    procedure PrunBitBtnBreakClick(Sender: TObject);
    procedure PrunBitBtnMinConfigClick(Sender: TObject);
    procedure PrunBitBtnMinCyclesClick(Sender: TObject);
    procedure PrunBitBtnPrimaryClick(Sender: TObject);
    procedure PrunBitBtnStartClick(Sender: TObject);
    procedure PrunSpinEditLimitChange(Sender: TObject);
    { TabSheet 'Recall' }
    procedure RecallBitBtnBreakClick(Sender: TObject);
    procedure RecallBitBtnDelRecallVectClick(Sender: TObject);
    procedure RecallBitBtnNewInitClick(Sender: TObject);
    procedure RecallBitBtnNewRecallVectClick(Sender: TObject);
    procedure RecallBitBtnOpenRecallSetClick(Sender: TObject);
    procedure RecallBitBtnRecallSetDimInpDecClick(Sender: TObject);
    procedure RecallBitBtnRecallSetDimInpIncClick(Sender: TObject);
    procedure RecallBitBtnSaveRecallSetClick(Sender: TObject);
    procedure RecallBitBtnStartClick(Sender: TObject);
    procedure RecallComboBoxRecallVectInpChange(Sender: TObject);
    procedure RecallComboBoxRecallVectOutpChange(Sender: TObject);
    procedure RecallEditRecallSetDimInpExit(Sender: TObject);
    procedure RecallEditRecallSetDimInpKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure RecallEditValInpCurrExit(Sender: TObject);
    procedure RecallEditValInpCurrKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RecallListBoxInpExit(Sender: TObject);
    procedure RecallListBoxInpKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { StatusBar variables }
    BackPropStatusImageIndex: Integer; // Index of current status image
    { TabSheet 'Evaluation' }
    EvalCurrCurve: Integer; // Number of current curve
    EvalLockUpChange: Boolean; // Avoid resetting lists/repainting diagrams
    EvalCurve: array[1..4] of TEvalCurve; // Storing data of current curve
    { TabSheet 'Learn' }
    { Lists storing LearnResults }
    LearnNetListNeuronWeights: TList; // Weights
    LearnNetListNeuronBias: TList; // Bias
    LearnNetListNeuronSlope: TList; // Slope
    LearnNetListNeuronWeightsDelta: TList; // Delta Weights
    LearnNetListNeuronBiasDelta: TList; // Delta Bias
    LearnNetListNeuronSlopeDelta: TList; // Delta Slope
    LearnNetListNeuronAdaptWeights: TList; // Adaptive Weights
    LearnNetListNeuronAdaptBias: TList; // Adaptive Bias
    LearnNetListNeuronAdaptSlope: TList; // Adaptive Slope
    LearnNetListNeuronNetInput: TList; // NeuronNetInput
    LearnNetListNeuronActivity: TList; // NeuronActivity
    LearnNetListNeuronOutput: TList; // NeuronOutput
    LearnNetListNeuronOutputDeriv: TList; // NeuronOutputDerivative
    LearnNetListNeuronDelta: TList; // NeuronDelta
    LearnNetListNeuronError: TList; // NeuronError
    LearnNetListNeuronActivityAll: TList; // Activities of all cycles (sum)
    LearnNetListNeuronPowerAll: TList; // Power of all cycles (sum)
    LearnNetListNeuronOutputAll: TList; // Ouput of all cycles (sum)
    LearnNetListNetGlobalError: TRealArray; // NetGlobalError
    LearnNetListNetLearnRateAdapt: TRealArray; // NetLearnRateRecursive
    { Parameters for diagrams }
    LearnErrorList: TRealArray; // GlobalError of net
    LearnErrorMinX, LearnErrorMaxX: Integer; // Minimum x-axis
    LearnErrorMinY, LearnErrorMaxY: Double; // Minimum y-axis
    LearnWeightsList: TRealArray; // Weights of neurons
    LearnWeightsMinX, LearnWeightsMaxX: Integer; // Minimum x-axis
    LearnWeightsMinY, LearnWeightsMaxY: Double; // Minimum y-axis
    LearnFactXDiag: Double; // X-Factor to match GlobalError data into diagram
    LearnFactYDiag: Double; // Y-Factor to match GlobalError data into diagram
    LearnFactYBar: Double; // Y-Factor to match Weights data into Bar-diagram
    LearnFactYHinton: Double; // Y-Factor to match Weights data into Hinton-diagram
    { TabSheet 'Pruning' }
    PrunNetListNetConfig: TList; // Configuration of optimized nets
    PrunNetListLearnCycles: TRealArray; // LearnCycles of optimized nets
    { Local lists within pruning algorithm }
    PrunLayerListNeuronSumWeights: TList;
    PrunLayerListLayerSumWeights: TRealArray;
    PrunLayerListNeuronPower: TList;
    PrunLayerListLayerSumPower: TRealArray;
    PrunLayerListNeuronProdRelWeightsPower: TList;
    PrunLayerListLayerMeanValProd: TRealArray;
    PrunNeuronListNeuronSumWeights: TRealArray;
    PrunNeuronListNeuronPower: TRealArray;
    PrunNeuronListNeuronProdRelWeightsPower: TRealArray;
    { Storing values of original net }
    PrunStoreLayerNo: Integer;
    PrunStoreInputNo: Integer;
    PrunStoreOutputNo: Integer;
    PrunNetMinStructure: Integer; // Number of optimal net concerning structure
    PrunNetMinCycles: Integer; // Number of optimal net concerning cycles
    procedure BackPropStatusBarShowHint(Sender: TObject); // Show hints in panel
    procedure BackPropSetStatus(ImageIndex: Integer; Text: string); // Set status in panel
    { TabSheet 'Configuration' }
    { Find function name for ComboBox }
    function ConfigFindFunctNameAct(FunctName: string): TFunctTypeAct;
    function ConfigFindFunctNameOutp(FunctName: string): TFunctTypeOutp;
    function ConfigFindFunctNameProp(FunctName: string): TFunctTypeProp;
    { TabSheet 'Evaluation' }
    procedure EvalDisableAll; // Disable all components of TabSheet
    procedure EvalEnableAll; // Enable all components of TabSheet
    procedure EvalGetDataList(Index: Integer; var List: TRealArray); // Get chosen data from LearnLists
    procedure EvalProcCorr(var EvalDataList: TRealArray;
      EvalDataListTemp: TRealArray; Depth: Integer); // DataProcessing correlation
    procedure EvalProcFilt(var EvalDataList: TRealArray; FiltConst: Double); // DataProcessing filter
    procedure EvalProcOper(var EvalDataList: TRealArray; // DataProcessing operations
      EvalDataListTemp: TRealArray; Choice: Integer; Adapt: Double);
    procedure EvalProcStat(var EvalDataList: TRealArray; // DataProcessing statistics
      Choice: Integer; Adapt: Double);
    procedure EvalSetDataDiag; // Prepare data for and paint diagram
    procedure EvalSetDataSingle; // Give out single chosen value
    { TabSheet 'Learn' }
    procedure LearnCollectData; // Collect LearnResults every given Intervall
    procedure LearnFreeAllLearnLists; // Free all lists containing LearnResults
    procedure LearnHideBoxes; // Hide GroupBoxes when displaying diagram during learning
    procedure LearnReadParams; // Read parameters of component BackPropNet and display them on form
    procedure LearnSetDataListGlobalErrorDuringLearning
      (ALearnErrorList: TRealArray); // Prepare data for diagram
    procedure LearnSetDataListsWeightsDuringLearning
      (ALearnWeightsList: TRealArray); // Prepare data for diagram
    procedure LearnShowBoxes; // Show GroupBoxes when no diagram to be displayed
    { TabSheet 'Pruning' }
    procedure PrunFreeAllPrunLists; // Free all lists containing results of pruning
  public
  end;

const
  crBackPropWaitCursor = 1; // Cursor used during ongoing actions

{ Declare function to export BackPropForm into the DLL 'BackPropagation' }
procedure CallBackPropagation; export;

implementation

uses
  { Self written units that are essential }
  { Diagram form for display of data on page 'Evaluation' }
  BackPropEvalDiagFormFile,
  { DataProcessing form to set parameters for correlation }
  BackPropEvalProcCorrFormFile,
  { DataProcessing form to set parameters for filter}
  BackPropEvalProcFiltFormFile,
  { DataProcessing form to set parameters for operations }
  BackPropEvalProcOperFormFile,
  { DataProcessing form to set parameters for statistics }
  BackPropEvalProcStatFormFile;

{$R *.DFM}
{$R BACKPROP.RES}

{ Define function to export BackPropForm into the DLL 'BackPropagation' }

procedure CallBackPropagation;
var
  BackPropForm: TBackPropForm;
begin
  try
    BackPropForm := TBackPropForm.Create(nil);
    try
      BackPropForm.ShowModal;
    finally
      BackPropForm.Free;
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg(E.ClassName + '(Fehler beim Erstellen des Formulars!)', mtError, [mbOk], 0);
    end;
  end;
end;

{ Section 'BackPropNet' }

procedure TBackPropForm.BackPropBackPropNetLearnExit(Sender: TBackPropNet;
  const ExitCode: TExitCode);
var
  HelpString: string;
begin
  if (LearnCheckBoxCollectResults.Checked = True) then begin
    { Paint last state of learning }
    LearnSetDataListGlobalErrorDuringLearning(LearnNetListNetGlobalError);
    with LearnNetListNeuronWeights do if (Count > 0) then
      LearnSetDataListsWeightsDuringLearning(TRealArray(TList(TList(
      Items[Count - 1]).Items[LearnSpinEditWeightChoiceLayer.Value - 2]).
      Items[LearnSpinEditWeightChoiceNeuron.Value - 1]));
    { Paint GlobalError Diagram }
    if (LearnSpeedButtonDiag.Down = True) then begin
      LearnPaintBoxDiagPaint(Self);
    end;
    { Paint Weights BarDiagram }
    if (LearnSpeedButtonBar.Down = True) then begin
      LearnPaintBoxBarPaint(Self);
    end;
    { Paint Weights HintonDiagram }
    if (LearnSpeedButtonHinton.Down = True) then begin
      LearnPaintBoxHintonPaint(Self);
    end;
  end;
  { Finishing announcement }
  HelpString := 'Keine.';
  if (ExitCode = exBreak) then HelpString := 'Abbruch durch Benutzer.';
  if (ExitCode = exError) then HelpString := 'Unterschreiten der angegebenen Fehlergrenze.';
  if (ExitCode = exCycle) then HelpString := 'Erreichen der vorgegebenen Anzahl an Lernschritten.';
  MessageBeep(0);
  MessageDlg('Lernen beendet. Ursache: ' + HelpString, mtInformation, [mbOk], 0);
end;

procedure TBackPropForm.BackPropBackPropNetLearnInterval(Sender: TBackPropNet);
begin
  if (LearnCheckBoxCollectResults.Checked = True) then begin
    { Call procedure to collect data: GlobalError etc. }
    LearnCollectData;
    { Set and edit all data to make them handy for diagrams }
    { Global error }
    LearnSetDataListGlobalErrorDuringLearning(LearnNetListNetGlobalError);
    { Weights }
    with LearnNetListNeuronWeights do
      LearnSetDataListsWeightsDuringLearning(TRealArray(TList(TList
      (Items[Count - 1]).Items[LearnSpinEditWeightChoiceLayer.Value - 2]).
      Items[LearnSpinEditWeightChoiceNeuron.Value - 1]));
    { Paint diagrams every 10 learn cycles to avoid flicker }
    if ((BackPropBackPropNet.Cycles mod 10) = 0) then begin
      { Global error diagram }
      if (LearnSpeedButtonDiag.Down = True) then LearnPaintBoxDiagPaint(Self);
      { Weights bar diagram }
      if (LearnSpeedButtonBar.Down = True) then LearnPaintBoxBarPaint(Self);
      { Weights Hinton diagram }
      if (LearnSpeedButtonHinton.Down = True) then LearnPaintBoxHintonPaint(Self);
    end;
  end;
end;

procedure TBackPropForm.BackPropBackPropNetRecallExit(Sender: TBackPropNet;
  const ExitCode: TExitCode);
var
  HelpString: string;
begin
  { Finishing announcement }
  HelpString := 'Keine.';
  if (ExitCode = exBreak) then HelpString := 'Abbruch durch Benutzer.';
  if (ExitCode = exCycle) then HelpString := 'Alle Vektoren des Sets erfolgreich berechnet.';
  MessageBeep(0);
  MessageDlg('Reproduzieren beendet. Ursache: ' + HelpString, mtInformation, [mbOk], 0);
end;

{ Section 'BackPropagation' }

procedure TBackPropForm.BackPropPageControlChange(Sender: TObject);
begin
  with BackPropPageControl do begin
    if (ActivePage = ConfigTabSheet) then HelpContext := 1;
    if (ActivePage = PattTabSheet) then HelpContext := 2;
    if (ActivePage = LearnTabSheet) then HelpContext := 3;
    if (ActivePage = EvalTabSheet) then HelpContext := 4;
    if (ActivePage = RecallTabSheet) then HelpContext := 5;
    if (ActivePage = PrunTabSheet) then HelpContext := 6;
  end;
end;

procedure TBackPropForm.BackPropSetStatus(ImageIndex: Integer; Text: string);
var
  HelpRect: TRect;
begin
  { Set index of current image }
  BackPropStatusImageIndex := ImageIndex;
  { Paint image }
  with BackPropStatusBar do begin
    with HelpRect do begin
      Left := 0;
      Top := 0;
      Right := Panels[0].Width;
      Bottom := Height;
    end;
    BackPropStatusBarDrawPanel(BackPropStatusBar, Panels[0], HelpRect);
    { Set text }
    Panels[1].Text := Text;
  end;
end;

procedure TBackPropForm.BackPropStatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
var
  Picture: TPicture;
begin
  Invalidate;
  { Overpaint old image with light grey }
  with StatusBar.Canvas do begin
    Brush.Color := clSilver;
    FillRect(Rect);
  end;
  { Paint new image }
  Picture := TPicture.Create;
  case BackPropStatusImageIndex of
    0: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusGreen');
    1: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusYellow');
    2: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusRed');
  end;
  StatusBar.Canvas.Draw(3, 3, TGraphic(Picture.Bitmap));
  Picture.Free;
end;

procedure TBackPropForm.BackPropStatusBarShowHint(Sender: TObject);
begin
  BackPropStatusBar.Panels[2].Text := Application.Hint;
end;

{ TabSheet 'Config' }

procedure TBackPropForm.ConfigBitBtnNewInitClick(Sender: TObject);
begin
  { Set net configuration to initial conditions }
  with BackPropBackPropNet do begin
    InputNo := 1;
    LayerNo := 1;
    Items[LayerNo - 1].NeuronNo := 1;
    { Fill components with net data }
    ConfigSpinEditNrLay.Value := (LayerNo + 1);
    ConfigSpinEditNrLayChange(Self);
  end;
  { Write FileName in Edit }
  ConfigEditFileCfg.Text := 'Untitled.cfg';
end;

procedure TBackPropForm.ConfigBitBtnOpenClick(Sender: TObject);
begin
  with BackPropNetFileDialog do try
    DialogMode := dmOpen;
    Title := 'ffnen einer KonfigurationsDatei';
    { Filter on Config }
    FilterIndex := 2;
    FileName := ConfigEditFileCfg.Text;
    DefaultExt := 'cfg';
    if (Execute = True) then begin
      BackPropBackPropNet.LoadFromFile(FileName);
      { Write FileName in ConfigEditFileCfg }
      ConfigEditFileCfg.Text := ExtractFileName(FileName);
    end;
    { Fill components with net data }
    ConfigSpinEditNrLay.Value := (BackPropBackPropNet.LayerNo + 1);
    ConfigSpinEditNrLayChange(Self);
    { Read values out of net component; write them into DialogComponents }
    LearnReadParams;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TBackPropForm.ConfigBitBtnSaveClick(Sender: TObject);
begin
  with BackPropNetFileDialog do try
    DialogMode := dmSave;
    Title := 'Speichern einer KonfigurationsDatei';
    { Filter on Config }
    FilterIndex := 2;
    FileName := ConfigEditFileCfg.Text;
    DefaultExt := 'cfg';
    if Execute then begin
      BackPropBackPropNet.SaveToFile(FileName);
      ConfigEditFileCfg.Text := ExtractFileName(FileName);
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TBackPropForm.ConfigComboBoxFunctActChange(Sender: TObject);
var
  i: Integer;
begin
  with ConfigComboBoxFunctAct do begin
    if ((ItemIndex = 0) or (ItemIndex = 1) or (ItemIndex = 2)) then begin
      MessageBeep(0);
      MessageDlg('Funktion existiert nicht im Fehlerrckfhrungs-Netz.', mtWarning, [mbOk], 0);
      ItemIndex := 3;
    end;
    for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
      if ConfigListBoxLay.Selected[i] then
        BackPropBackPropNet.Items[i - 1].ActFunction := ConfigFindFunctNameAct(Items[ItemIndex]);
  end;
end;

procedure TBackPropForm.ConfigComboBoxFunctOutpChange(Sender: TObject);
var
  i: Integer;
begin
  with ConfigComboBoxFunctOutp do begin
    if ((ItemIndex = 3) or (ItemIndex = 4) or (ItemIndex = 5)) then begin
      MessageBeep(0);
      MessageDlg('Funktion existiert nicht im Fehlerrckfhrungs-Netz.', mtWarning, [mbOk], 0);
      ItemIndex := 1;
    end;
    for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
      if ConfigListBoxLay.Selected[i] then
        BackPropBackPropNet.Items[i - 1].OutpFunction := ConfigFindFunctNameOutp(Items[ItemIndex]);
  end;
end;

procedure TBackPropForm.ConfigComboBoxRulePropChange(Sender: TObject);
var
  i: Integer;
begin
  with ConfigComboBoxRuleProp do begin
    if (ItemIndex = 0) then begin
      MessageBeep(0);
      MessageDlg('Funktion existiert nicht im Fehlerrckfhrungs-Netz.', mtWarning, [mbOk], 0);
      ItemIndex := 1;
    end;
    for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
      if ConfigListBoxLay.Selected[i] then
        BackPropBackPropNet.Items[i - 1].PropFunction := ConfigFindFunctNameProp(Items[ItemIndex]);
  end;
end;

procedure TBackPropForm.ConfigEditFunctActFactScalExit(Sender: TObject);
var
  i: Integer;
begin
  with ConfigEditFunctActFactScal do try
    for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
      if ConfigListBoxLay.Selected[i] then
        BackPropBackPropNet.Items[i - 1].Scale := StrToFloat(Text);
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Skalierungsfaktor prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Scale);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.ConfigEditFunctActFactScalKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then ConfigEditFunctActFactScalExit(Self);
end;

procedure TBackPropForm.ConfigEditFunctOutpMaxExit(Sender: TObject);
var
  HelpValue: Double;
  i: Integer;
begin
  with ConfigEditFunctOutpMax do try
    HelpValue := StrToFloat(Text);
    { Check if OutpMax is greater than OutpMin in OutputFunction }
    if (HelpValue <= StrToFloat(ConfigEditFunctOutpMin.Text)) then begin
      MessageBeep(0);
      MessageDlg('Obergrenze muss grsser sein als untere Grenze!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].OutputMax);
      SetFocus;
    end
    else for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
      if ConfigListBoxLay.Selected[i] then
        BackPropBackPropNet.Items[i - 1].OutputMax := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Maximum prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].OutputMax);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.ConfigEditFunctOutpMaxKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then ConfigEditFunctOutpMaxExit(Self);
end;

procedure TBackPropForm.ConfigEditFunctOutpMinExit(Sender: TObject);
var
  HelpValue: Double;
  i: Integer;
begin
  with ConfigEditFunctOutpMin do try
    HelpValue := StrToFloat(Text);
    { Check if OutpMax is greater than OutpMin in OutputFunction }
    if (HelpValue >= StrToFloat(ConfigEditFunctOutpMax.Text)) then begin
      MessageDlg('Obergrenze muss grsser sein als untere Grenze!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].OutputMin);
      SetFocus;
    end
    else for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
      if ConfigListBoxLay.Selected[i] then
        BackPropBackPropNet.Items[i - 1].OutputMin := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Minimum prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].OutputMin);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.ConfigEditFunctOutpMinKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then ConfigEditFunctOutpMinExit(Self);
end;

procedure TBackPropForm.ConfigEditFunctOutpSlopeExit(Sender: TObject);
var
  i: Integer;
begin
  if (LearnCheckBoxLearnSlope.Checked = True) then begin
    MessageDlg('Wert fr Anstieg kann nicht gesetzt werden, solange dieser ' +
      'zum Lernen aktiviert ist (Seite "Lernen")!', mtWarning, [mbOk], 0);
    with ConfigEditFunctOutpSlope do
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Slope);
    ConfigEditFunctOutpMin.SetFocus;
  end
  else begin
    with ConfigEditFunctOutpSlope do try
      for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
        if ConfigListBoxLay.Selected[i] then
          BackPropBackPropNet.Items[i - 1].Slope := StrToFloat(Text);
    except
      on EConvertError do begin
        MessageBeep(0);
        MessageDlg('Wert fr Anstieg prfen (Fliesskommawert)!', mtError, [mbOk], 0);
        Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Slope);
        SetFocus;
      end;
    end;
  end;
end;

procedure TBackPropForm.ConfigEditFunctOutpSlopeKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then ConfigEditFunctOutpSlopeExit(Self);
end;

procedure TBackPropForm.ConfigEditFunctOutpThreshExit(Sender: TObject);
var
  i: Integer;
begin
  if (LearnCheckBoxLearnThresh.Checked = True) then begin
    MessageBeep(0);
    MessageDlg('Wert fr Schwelle kann nicht gesetzt werden, solange diese ' +
      'zum Lernen aktiviert ist (Seite "Lernen")!', mtWarning, [mbOk], 0);
    with ConfigEditFunctOutpThresh do
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Bias);
    ConfigEditFunctOutpSlope.SetFocus;
  end
  else begin
    with ConfigEditFunctOutpThresh do try
      for i := 1 to (ConfigListBoxLay.Items.Count - 1) do
        if ConfigListBoxLay.Selected[i] then
          BackPropBackPropNet.Items[i - 1].Bias := StrToFloat(Text);
    except
      on EConvertError do begin
        MessageBeep(0);
        MessageDlg('Wert fr Schwellenwert prfen (Fliesskommawert)!',
        mtError, [mbOk], 0);
        Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Bias);
        SetFocus;
      end;
    end;
  end;
end;

procedure TBackPropForm.ConfigEditFunctOutpThreshKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then ConfigEditFunctOutpThreshExit(Self);
end;

function TBackPropForm.ConfigFindFunctNameAct(FunctName: string): TFunctTypeAct;
begin
  Result := faLinear;
  { Activation Function }
  if FunctName = 'Linear' then Result := faLinear;
  if FunctName = 'BSB' then Result := faBSB;
  if FunctName = 'DMA' then Result := faDMA;
  if FunctName = 'Hopfield' then Result := faHopfield;
end;

function TBackPropForm.ConfigFindFunctNameOutp(FunctName: string): TFunctTypeOutp;
begin
  Result := foFermi;
  { Output Function }
  if FunctName = 'Fermi' then Result := foFermi;
  if FunctName = 'Tangens Hyperbolicus' then Result := foTanh;
  if FunctName = 'Sinus (Cosinus-Quetsch)' then Result := foSquashCos;
  if FunctName = 'Arcus Tangens' then Result := foArcTan;
  if FunctName = 'Linear' then Result := foLinear;
  if FunctName = 'Schwellenwert' then Result := foThreshold;
  if FunctName = 'Linearer Schwellenwert' then Result := foThresholdLin;
  if FunctName = 'Rampe' then Result := foHardLimit;
end;

function TBackPropForm.ConfigFindFunctNameProp(FunctName: string): TFunctTypeProp;
begin
  Result := fpScalProd;
  { Propagation Rule }
  if FunctName = 'SkalarProdukt' then Result := fpScalProd;
  if FunctName = 'Sigma-Pi' then Result := fpSigmaPi;
end;

procedure TBackPropForm.ConfigListBoxLayClick(Sender: TObject);
var
  i: Integer;
begin
  { Setting indices the same for both of the list boxes }
  ConfigListBoxNrNeur.ItemIndex := ConfigListBoxLay.ItemIndex;
  { Filling all components with parameters taken from the net component }
  { Excluding input layer, i.e. number of inputs to the net}
  if (ConfigListBoxLay.ItemIndex > 0) then begin
    { Set value for number of neurons }
    with ConfigSpinEditNrNeur do
      Value := BackPropBackPropNet.Items[ConfigListBoxLay.ItemIndex - 1].NeuronNo;
    { Set value of propagation rule }
    with ConfigComboBoxRuleProp do begin
      Enabled := True;
      Color := clWindow;
      Font.Color := clBlack;
      for i := 0 to (Items.Count - 1) do
        if (ConfigFindFunctNameProp(Items[i]) = BackPropBackPropNet.Items[
          (ConfigListBoxLay.ItemIndex - 1)].PropFunction) then ItemIndex := i;
    end;
    { Set value of activation function }
    with ConfigComboBoxFunctAct do begin
      Enabled := True;
      Color := clWindow;
      Font.Color := clBlack;
      for i := 0 to (Items.Count - 1) do
        if (ConfigFindFunctNameAct(Items[i]) = BackPropBackPropNet.Items[
          (ConfigListBoxLay.ItemIndex - 1)].ActFunction) then ItemIndex := i;
    end;
    { Set value of scale factor of activation function }
    with ConfigEditFunctActFactScal do begin
      Enabled := True;
      Color := clWindow;
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Scale);
    end;
    { Set value of output function }
    with ConfigComboBoxFunctOutp do begin
      Enabled := True;
      Color := clWindow;
      Font.Color := clBlack;
      for i := 0 to (Items.Count - 1) do
        if (ConfigFindFunctNameOutp(Items[i]) = BackPropBackPropNet.Items[
          (ConfigListBoxLay.ItemIndex - 1)].OutpFunction) then ItemIndex := i;
    end;
    { Set value of threshold = bias of output function }
    with ConfigEditFunctOutpThresh do begin
      Enabled := True;
      Color := clWindow;
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Bias);
    end;
    { Set value of slope of output function }
    with ConfigEditFunctOutpSlope do begin
      Enabled := True;
      Color := clWindow;
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].Slope);
    end;
    { Set value of minimum of output function }
    with ConfigEditFunctOutpMin do begin
      Enabled := True;
      Color := clWindow;
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].OutputMin);
    end;
    { Set value of maximum of output function }
    with ConfigEditFunctOutpMax do begin
      Enabled := True;
      Color := clWindow;
      Text := FloatToStr(BackPropBackPropNet.Items[(ConfigListBoxLay.ItemIndex - 1)].OutputMax);
    end;
  end
  { Section for the input layer = number of net inputs }
  else begin
    { Set value for number of neurons = net inputs }
    with ConfigSpinEditNrNeur do
      Value := BackPropBackPropNet.InputNo;
    { Set no value for propagation rule }
    with ConfigComboBoxRuleProp do begin
      Enabled := False;
      Color := clBtnFace;
      Font.Color := clBtnFace;
    end;
    { Set no value for activation function }
    with ConfigComboBoxFunctAct do begin
      Enabled := False;
      Color := clBtnFace;
      Font.Color := clBtnFace;
    end;
    { Set no value for scale factor of activation function }
    with ConfigEditFunctActFactScal do begin
      Enabled := False;
      Color := clBtnFace;
    end;
    { Set no value for output function }
    with ConfigComboBoxFunctOutp do begin
      Enabled := False;
      Color := clBtnFace;
      Font.Color := clBtnFace;
    end;
    { Set no value for threshold = bias of output function }
    with ConfigEditFunctOutpThresh do begin
      Enabled := False;
      Color := clBtnFace;
    end;
    { Set no value for slope of output function }
    with ConfigEditFunctOutpSlope do begin
      Enabled := False;
      Color := clBtnFace;
    end;
    { Set no value for minimum of output function }
    with ConfigEditFunctOutpMin do begin
      Enabled := False;
      Color := clBtnFace;
    end;
    { Set no value for maximum of output function }
    with ConfigEditFunctOutpMax do begin
      Enabled := False;
      Color := clBtnFace;
    end;
  end;
end;

procedure TBackPropForm.ConfigSpinEditNrLayChange(Sender: TObject);
var
  i: Integer;
  HelpValue: Integer;
begin
  { Check if string is value }
  with ConfigSpinEditNrLay do try
    HelpValue := Value;
    { Set LayerNo of net component }
    BackPropBackPropNet.LayerNo := (HelpValue - 1);
    { Write layer data in listboxes }
    ConfigListBoxLay.Clear;
    ConfigListBoxNrNeur.Clear;
    with BackPropBackPropNet do
      for i := 0 to (HelpValue - 1) do
        if (i = 0) then begin
          ConfigListBoxLay.Items.Add('EingangsSchicht ' + IntToStr(i + 1));
          ConfigListBoxNrNeur.Items.Add(IntToStr(InputNo));
        end
        else begin
          if (i = (HelpValue - 1)) then begin
            ConfigListBoxLay.Items.Add('AusgangsSchicht ' + IntToStr(i + 1));
            ConfigListBoxNrNeur.Items.Add(IntToStr(Items[(i - 1)].NeuronNo));
          end
          else begin
            ConfigListBoxLay.Items.Add('VerborgeneSchicht ' + IntToStr(i + 1));
            ConfigListBoxNrNeur.Items.Add(IntToStr(Items[(i - 1)].NeuronNo));
          end;
        end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Schichtanzahl muss Integer sein!', mtError, [mbOk], 0);
      Value := (BackPropBackPropNet.LayerNo + 1);
      SetFocus;
    end;
  end;
  ConfigListBoxLay.ItemIndex := 0;
  ConfigListBoxLayClick(self);
  EvalDisableAll;
end;

procedure TBackPropForm.ConfigSpinEditNrNeurExit(Sender: TObject);
var
  HelpValue: Integer;
  i: Integer;
begin
  { Check if input is value }
  with ConfigSpinEditNrNeur do try
    HelpValue := Value;
    { For all layers but input layer }
    for i := 0 to (ConfigListBoxLay.Items.Count - 1) do begin
      if ConfigListBoxLay.Selected[i] then begin
        if (i = 0) then begin
          { For input layer = net inputs }
          BackPropBackPropNet.InputNo := HelpValue;
          { Set value for neuron number new in ListBox }
          ConfigListBoxNrNeur.Items[0] := IntToStr(HelpValue);
        end
        else begin
          { Set NeuronNo of currently chosen layers in net component }
          BackPropBackPropNet.Items[i - 1].NeuronNo := HelpValue;
          { Set values for neuron number new in listbox }
          ConfigListBoxNrNeur.Items[i] := IntToStr(HelpValue);
        end;
      end;
    end;
    ConfigListBoxNrNeur.ItemIndex := ConfigListBoxLay.ItemIndex;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Anzahl der Neuronen muss Integer sein!', mtError, [mbOk], 0);
      if (ConfigListBoxLay.ItemIndex > 0) then
        Value := BackPropBackPropNet.Items[ConfigListBoxLay.ItemIndex - 1].NeuronNo
      else Value := BackPropBackPropNet.InputNo;
      SetFocus;
    end;
  end;
  EvalDisableAll;
end;

procedure TBackPropForm.ConfigSpinEditNrNeurKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then ConfigSpinEditNrNeurExit(Self);
end;

{ TabSheet 'Evaluation' following }

procedure TBackPropForm.EvalBitBtnNewInitClick(Sender: TObject);
var
  i: Integer;
begin
  with BackPropBackPropNet do
    if (Cycles < Interval) then begin
      MessageBeep(0);
      raise Exception.Create('Netz erst erfolgreich anlernen, ' +
        'bevor Lerndaten ausgewertet werden knnen!');
    end
    else begin
      { Fill records storing curve information }
      for i := 1 to 4 do
        with EvalCurve[i] do begin
          CurrData := (EvalRadioGroupData.Items.Count - 1);
          Proc.ProcType := 0;
          Layer := 2;
          Neuron := 1;
          Weight := 1;
          Cycle := 1;
        end;
      { Press SpeedButton for first curve to activate }
      EvalCurveSpeedButton1.Down := True;
      EvalCurveSpeedButtonClick(Self);
      { Set labels of curve-buttons }
      EvalCurveLabel1.Caption := 'Leer';
      EvalCurveLabel2.Caption := 'Leer';
      EvalCurveLabel3.Caption := 'Leer';
      EvalCurveLabel4.Caption := 'Leer';
      { Set Value and Min/MaxValue of SpinEditLayer }
      with EvalSpinEditLayer do begin
        MaxValue := BackPropBackPropNet.LayerNo + 1;
        if (MaxValue > 2) then MinValue := 2
        else MinValue := 1;
      end;
      EvalSpinEditLayerChange(Self);
      { Set Value and Min/MaxValue of SpinEditCycle displaying single values }
      with EvalSpinEditCycle do begin
        MaxValue := BackPropBackPropNet.Cycles;
        if (MaxValue > 1) then MinValue := 1
        else MinValue := 0;
      end;
    end;
end;

procedure TBackPropForm.EvalBitBtnPrintClick(Sender: TObject);
var
  SizeRatio: Double;
  SourceRect: TRect;
  DestRect: TRect;
begin
  with BackPropEvalDiagForm do try
    if (BackPropPrintDialog.Execute = True) then begin
      with EvalDiagPaintBox1 do begin
        { Calculate ratio of images' height to width - for all 4 diags equal }
        SizeRatio := (Height / Width);
        { Get size of diagram from canvas of EvalPaintBox }
        SourceRect := Rect(0, 0, Width, Height);
      end;
      with Printer do begin
        { Get the various sizes for the printer's canvas }
        DestRect := Rect((0 + 400), (0 + 300), (PageWidth - 300), (Trunc(
          (PageWidth - 300) * SizeRatio) - GetDeviceCaps(Handle, LOGPIXELSX)));
        { Text appearing on the PrintManager and on networked pages }
        Title := 'Res Neuronum - BackPropagationNetwork: printing diagram';
        { Start the print job }
        BeginDoc;
        with DestRect do begin
          { Draw a rectangle to show the margins }
          Canvas.Rectangle(Left, Top, Right, Bottom);
          { Decrease size of rectangle so image doesn't draw over it }
          DestRect := Rect((Left + 5), (Top + 5), (Right - 5), Bottom - 5);
        end;
        case EvalCurrCurve of
          { Send contents of SourceRect to DestRect on printer }
          1: Canvas.CopyRect(DestRect, EvalDiagPaintBox1.Canvas, SourceRect);
          2: Canvas.CopyRect(DestRect, EvalDiagPaintBox2.Canvas, SourceRect);
          3: Canvas.CopyRect(DestRect, EvalDiagPaintBox3.Canvas, SourceRect);
          4: Canvas.CopyRect(DestRect, EvalDiagPaintBox4.Canvas, SourceRect);
        end;
        { Stop the print job }
        EndDoc;
      end;
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht drucken: ' + E.Message, mtError, [mbOk], 0);
    end;
  end;
end;

procedure TBackPropForm.EvalBitBtnPrintAllClick(Sender: TObject);
begin
  if (BackPropPrintDialog.Execute = True) then begin
    { Text appearing on PrintManager and on networked pages }
    Printer.Title := 'Res Neuronum - BackPropagationNetwork: printing diagram form';
    BackPropEvalDiagForm.Print;
  end;
end;

procedure TBackPropForm.EvalCurveSpeedButtonClick(Sender: TObject);
var
  DummyValue: Integer;
begin
  { Lock up drawing diagrams to avoid troubles while setting new values }
  EvalLockUpChange := True;
  if (EvalCurveSpeedButton1.Down = True) then EvalCurrCurve := 1;
  if (EvalCurveSpeedButton2.Down = True) then EvalCurrCurve := 2;
  if (EvalCurveSpeedButton3.Down = True) then EvalCurrCurve := 3;
  if (EvalCurveSpeedButton4.Down = True) then EvalCurrCurve := 4;
  { Set layer, neuron, weight }
  with EvalCurve[EvalCurrCurve] do begin
    DummyValue := Layer;
    EvalSpinEditLayer.Value := DummyValue;
    DummyValue := Neuron;
    EvalSpinEditNeuron.Value := DummyValue;
    DummyValue := Weight;
    EvalSpinEditWeight.Value := DummyValue;
    { Set cycle }
    DummyValue := Cycle;
    { If equal: click necessary to print value; if unequal: clicks itself }
    if (DummyValue = EvalSpinEditCycle.Value) then EvalSpinEditCycleChange(Self)
    else EvalSpinEditCycle.Value := DummyValue;
    { Set RadioGroup index }
    DummyValue := CurrData;
    { If equal: click necessary to disable/gray boxes; if unequal: clicks itself }
    if (DummyValue = EvalRadioGroupData.ItemIndex) then EvalRadioGroupDataClick(Self)
    else EvalRadioGroupData.ItemIndex := DummyValue;
    { Set data processing type }
    DummyValue := Proc.ProcType;
    case DummyValue of
      0: EvalProcSpeedButton0.Down := True;
      1: EvalProcSpeedButton1.Down := True;
      2: EvalProcSpeedButton2.Down := True;
      3: EvalProcSpeedButton3.Down := True;
      4: EvalProcSpeedButton4.Down := True;
      5: EvalProcSpeedButton5.Down := True;
      6: EvalProcSpeedButton6.Down := True;
      7: EvalProcSpeedButton7.Down := True;
      8: EvalProcSpeedButton8.Down := True;
    end;
  end;
  { Cancel lock up to enable drawing diagrams }
  EvalLockUpChange := False;
end;

procedure TBackPropForm.EvalDisableAll;
begin
  EvalBitBtnNewInit.Enabled := False;
  EvalGroupBoxCurve.Enabled := False;
  EvalGroupBoxElement.Enabled := False;
  EvalGroupBoxSingleValue.Enabled := False;
  EvalRadioGroupData.Enabled := False;
  EvalGroupBoxDataProc.Enabled := False;
  EvalBitBtnPrint.Enabled := False;
  EvalBitBtnPrintAll.Enabled := False;
  EvalSpeedButtonShowDiag.Enabled := False;
end;

procedure TBackPropForm.EvalEnableAll;
begin
  EvalBitBtnNewInit.Enabled := True;
  EvalGroupBoxCurve.Enabled := True;
  EvalGroupBoxElement.Enabled := True;
  EvalGroupBoxSingleValue.Enabled := True;
  EvalRadioGroupData.Enabled := True;
  EvalGroupBoxDataProc.Enabled := True;
  EvalBitBtnPrint.Enabled := True;
  EvalBitBtnPrintAll.Enabled := True;
  EvalSpeedButtonShowDiag.Enabled := True;
end;

procedure TBackPropForm.EvalGetDataList(Index: Integer; var List: TRealArray);
var
  i: Integer;
begin
  with EvalCurve[EvalCurrCurve] do
    case Index of
      0: for i := 0 to (LearnNetListNeuronWeights.Count - 1) do
        List.Add(TRealArray(TList(TList(LearnNetListNeuronWeights.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]).Items[Weight - 1]);
      1: for i := 0 to (LearnNetListNeuronBias.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronBias.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      2: for i := 0 to (LearnNetListNeuronSlope.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronSlope.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      3: for i := 0 to (LearnNetListNeuronWeightsDelta.Count - 1) do
        List.Add(TRealArray(TList(TList(LearnNetListNeuronWeightsDelta.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]).Items[Weight - 1]);
      4: for i := 0 to (LearnNetListNeuronBiasDelta.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronBiasDelta.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      5: for i := 0 to (LearnNetListNeuronSlopeDelta.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronSlopeDelta.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      6: for i := 0 to (LearnNetListNeuronAdaptWeights.Count - 1) do
        List.Add(TRealArray(TList(TList(LearnNetListNeuronAdaptWeights.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]).Items[Weight - 1]);
      7: for i := 0 to (LearnNetListNeuronAdaptBias.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronAdaptBias.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      8: for i := 0 to (LearnNetListNeuronAdaptSlope.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronAdaptSlope.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      9: for i := 0 to (LearnNetListNeuronNetInput.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronNetInput.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      10: for i := 0 to (LearnNetListNeuronActivity.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronActivity.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      11: for i := 0 to (LearnNetListNeuronOutput.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronOutput.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      12: for i := 0 to (LearnNetListNeuronOutputDeriv.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronOutputDeriv.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      13: for i := 0 to (LearnNetListNeuronDelta.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronDelta.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      14: for i := 0 to (LearnNetListNeuronError.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronError.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      15: for i := 0 to (LearnNetListNeuronActivityAll.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronActivityAll.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      16: for i := 0 to (LearnNetListNeuronPowerAll.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronPowerAll.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      17: for i := 0 to (LearnNetListNeuronOutputAll.Count - 1) do
        List.Add(TRealArray(TList(LearnNetListNeuronOutputAll.Items[i]).
        Items[Layer - 2]).Items[Neuron - 1]);
      18: for i := 0 to (LearnNetListNetGlobalError.Count - 1) do
        List.Add(LearnNetListNetGlobalError.Items[i]);
      19: for i := 0 to (LearnNetListNetLearnRateAdapt.Count - 1) do
        List.Add(LearnNetListNetLearnRateAdapt.Items[i]);
      20: { No list was chosen }
    end;
end;

procedure TBackPropForm.EvalProcCorr(var EvalDataList: TRealArray;
  EvalDataListTemp: TRealArray; Depth: Integer);
var
  EvalProcCorrList: TRealArray;
  i, MaxLength, n: Integer;
  Sum, Y, Y0: Double;
begin
  { Create local list }
  EvalProcCorrList := TRealArray.Create(0);
  with EvalDataList do begin
    { Get length of shorter list }
    if (Count < EvalDataListTemp.Count) then MaxLength := Count
    else MaxLength := EvalDataListTemp.Count;
    { Correlate data }
    Y0 := 1;
    for n := 0 to (Depth - 1) do begin
      Sum := 0;
      for i := 0 to ((MaxLength - 1) - n) do
        Sum := (Sum + (Items[i] * EvalDataListTemp.Items[i + n]));
      Y := (1 / (MaxLength - n) * Sum);
      if (n = 0) then Y0 := Y;
      { Normalizing on first value }
      try
        Y := (Y / Y0);
      except
        on Exception do Y := 0;
      end;
      EvalProcCorrList.Add(Y);
    end;
    { Write results into EvalDataList }
    DeleteAll;
    for i := 0 to (EvalProcCorrList.Count - 1) do Add(EvalProcCorrList.Items[i]);
  end;
  { Free local list }
  EvalProcCorrList.Free;
end;

procedure TBackPropForm.EvalProcFilt(var EvalDataList: TRealArray;
  FiltConst: Double);
var
  EvalProcFiltList: TRealArray;
  i: Integer;
  FiltValue: Double;
begin
  { Create local list }
  EvalProcFiltList := TRealArray.Create(0);
  with EvalDataList do begin
    { Apply filter on data }
    FiltValue := 0;
    for i := 0 to (Count - 1) do begin
      FiltValue := ((FiltConst * FiltValue) + ((1 - FiltConst) * Items[i]));
      EvalProcFiltList.Add(FiltValue);
    end;
    { Write results in EvalDataList }
    DeleteAll;
    for i := 0 to (EvalProcFiltList.Count - 1) do Add(EvalProcFiltList.Items[i]);
  end;
  { Free local list }
  EvalProcFiltList.Free;
end;

procedure TBackPropForm.EvalProcOper(var EvalDataList: TRealArray;
  EvalDataListTemp: TRealArray; Choice: Integer; Adapt: Double);
var
  EvalProcOperList: TRealArray;
  i, MaxLength: Integer;
  Sum: Double;
begin
  { Create local list }
  EvalProcOperList := TRealArray.Create(0);
  with EvalDataList do begin
    { Get length of shorter list }
    if (Count < EvalDataListTemp.Count) then MaxLength := Count
    else MaxLength := EvalDataListTemp.Count;
    { Apply chosen operation }
    Sum := 0;
    for i := 0 to (MaxLength - 1) do begin
      case Choice of
        0: Sum := (Sum - Adapt * (Sum - (Items[i] + EvalDataListTemp.Items[i])));
        1: Sum := (Sum - Adapt * (Sum - (Abs(Items[i]) + Abs(EvalDataListTemp.Items[i]))));
        2: Sum := (Sum - Adapt * (Sum - (Items[i] * EvalDataListTemp.Items[i])));
      end;
      EvalProcOperList.Add(Sum);
    end;
    { Write results into EvalDataList }
    DeleteAll;
    for i := 0 to (EvalProcOperList.Count - 1) do Add(EvalProcOperList.Items[i]);
  end;
  { Free local list }
  EvalProcOperList.Free;
end;

procedure TBackPropForm.EvalProcSpeedButtonClick(Sender: TObject);
var
  EvalDataList: TRealArray;
begin
  with EvalCurve[EvalCurrCurve] do begin
    { Data processing off }
    if (EvalProcSpeedButton0.Down = True) then
      { Write data in CurveRecord }
      Proc.ProcType := 0;
    { DataProcessing 1: Correlation }
    if (EvalProcSpeedButton1.Down = True) then try
      { Create form to choose parameters }
      BackPropEvalProcCorrForm := TBackPropEvalProcCorrForm.Create(Self);
      with BackPropEvalProcCorrForm do begin
        with EvalProcCorrComboBoxWith do begin
          Items.Assign(EvalRadioGroupData.Items);
          ItemIndex := CurrData;
        end;
        { Get local list to count length for setting correlation depth }
        EvalDataList := TRealArray.Create(0);
        EvalGetDataList(CurrData, EvalDataList);
        if (EvalDataList.Count > 1) then begin
          { Set min/max-values for CorrSpinEditDepth }
          with EvalProcCorrSpinEditDepth do begin
            MaxValue := (EvalDataList.Count);
            if (MaxValue > 1) then MinValue := 1 else MinValue := 0;
            { Write correlation depth in SpinEditDepth and initialize depth }
            Value := (EvalDataList.Count);
          end;
        end;
        { Free local list }
        EvalDataList.Free;
        { Get result of modal form }
        if (ShowModal = mrOk) then begin
          { Write data in CurveRecord }
          Proc.ProcType := 1;
          Proc.CorrWith := EvalProcCorrComboBoxWith.ItemIndex;
          Proc.CorrDepth := EvalProcCorrSpinEditDepth.Value;
          if (CurrData = EvalProcCorrComboBoxWith.ItemIndex) then
            Proc.CorrName := 'AutoKorrelation'
          else Proc.CorrName := 'KreuzKorrelation';
        end;
      end;
    finally
      BackPropEvalProcCorrForm.Free;
    end;
    { DataProcessing 2: Statistics }
    if (EvalProcSpeedButton2.Down = True) then try
      { Create form to choose parameters }
      BackPropEvalProcStatForm := TBackPropEvalProcStatForm.Create(Self);
      with BackPropEvalProcStatForm do
        { Get result of modal form }
        if (ShowModal = mrOk) then begin
          { Write data in CurveRecord }
          Proc.ProcType := 2;
          Proc.StatMoment := EvalProcStatRadioGroup.ItemIndex;
          Proc.StatMomentName := EvalProcStatRadioGroup.Items[EvalProcStatRadioGroup.ItemIndex];
          Proc.StatAdaptConst := StrToFloat(EvalProcStatEditAdaptConst.Text);
        end;
    finally
      BackPropEvalProcStatForm.Free;
    end;
    { DataProcessing 3: Filt }
    if (EvalProcSpeedButton3.Down = True) then try
      { Create form to choose parameters }
      BackPropEvalProcFiltForm := TBackPropEvalProcFiltForm.Create(Self);
      with BackPropEvalProcFiltForm do
        { Get result of modal form }
        if (ShowModal = mrOk) then begin
          { Write data in CurveRecord }
          Proc.ProcType := 3;
          Proc.FiltConst := StrToFloat(EvalProcFiltEdit.Text);
          if (EvalProcFiltSpeedButtonHighPass.Down = True) then
            Proc.FiltName := 'HochPass'
          else Proc.FiltName := 'TiefPass';
        end;
    finally
      { Free form }
      BackPropEvalProcFiltForm.Free;
    end;
    { DataProcessing 4: Oper }
    if (EvalProcSpeedButton4.Down = True) then try
      { Create form to choose parameters }
      BackPropEvalProcOperForm := TBackPropEvalProcOperForm.Create(Self);
      with BackPropEvalProcOperForm do begin
        EvalProcOperComboBox1.ItemIndex := 0;
        EvalProcOperComboBox2.Items.Assign(EvalRadioGroupData.Items);
        EvalProcOperComboBox2.ItemIndex := CurrData;
        { Get result of modal form }
        if (ShowModal = mrOk) then begin
          { Write data in CurveRecord }
          Proc.ProcType := 4;
          Proc.OperWith := EvalProcOperComboBox2.ItemIndex;
          Proc.OperKind := EvalProcOperComboBox1.ItemIndex;
          if (EvalProcOperComboBox1.ItemIndex = 0) then Proc.OperName := 'Summe';
          if (EvalProcOperComboBox1.ItemIndex = 1) then Proc.OperName := 'Summe Betrge';
          if (EvalProcOperComboBox1.ItemIndex = 2) then Proc.OperName := 'Produkt';
          Proc.OperAdaptConst := StrToFloat(EvalProcOperEditAdaptConst.Text);
        end;
      end;
    finally
      { Free form }
      BackPropEvalProcOperForm.Free;
    end;
    { DataProcessing 5 }
    if (EvalProcSpeedButton5.Down = True) then
      { Write data in CurveRecord }
      Proc.ProcType := 5;
    { DataProcessing 6 }
    if (EvalProcSpeedButton6.Down = True) then
      { Write data in CurveRecord }
      Proc.ProcType := 6;
    { DataProcessing 7 }
    if (EvalProcSpeedButton7.Down = True) then
      { Write data in CurveRecord }
      Proc.ProcType := 7;
    { DataProcessing 8 }
    if (EvalProcSpeedButton8.Down = True) then
      { Write data in CurveRecord }
      Proc.ProcType := 8;
  end;
  if (EvalLockUpChange = False) then EvalSetDataDiag;
end;

procedure TBackPropForm.EvalProcStat(var EvalDataList: TRealArray;
  Choice: Integer; Adapt: Double);
var
  EvalProcStatList: TRealArray;
  i: Integer;
  Mean, MeanSquVal, OldY, PrevValue, Result: Double;
begin
  { Create local list }
  EvalProcStatList := TRealArray.Create(0);
  with EvalDataList do begin
    { Initialize values }
    Mean := 0;
    MeanSquVal := 0;
    PrevValue := 0;
    Result := 0;
    for i := 0 to (Count - 1) do begin
      if (i > 0) then PrevValue := Items[i - 1];
      { Mean }
      Mean := (Mean + (Adapt * (Items[i] - Mean)));
      { Mean squared value }
      MeanSquVal := (MeanSquVal + (Adapt * (Sqr(Items[i]) - MeanSquVal)));
      case choice of
        0: { Mean (normal moment) }
          Result := Mean;
        1: { Mean Squared Value = effective Value (normal moment) }
          Result := MeanSquVal;
        2: { Dispersion = Variance (central moment) }
          Result := (Result + (Adapt * (Sqr(Items[i] - Mean) - Result)));
        3: { Mean of absolute dispersion }
          Result := (Result - (Adapt * (Result - Abs(Items[i] - Mean))));
        4: begin
          { Extent of dynamics }
          if (i > 0) then OldY := PrevValue else OldY := Items[i];
          Result := (Result - (Adapt * (Result - Abs(Items[i] - OldY))));
        end;
        5: begin
          { Extent of noise }
          if (i > 0) then OldY := PrevValue else OldY := Items[i];
          Result := (Result - (Adapt * (Result - ((Items[i] - OldY) * (Items[i] - OldY)))));
        end;
      end;
      EvalProcStatList.Add(Result);
    end;
    DeleteAll;
    for i := 0 to (EvalProcStatList.Count - 1) do Add(EvalProcStatList.Items[i]);
  end;
  { Free local list }
  EvalProcStatList.Free;
end;

procedure TBackPropForm.EvalRadioGroupDataClick(Sender: TObject);
begin
  with EvalCurve[EvalCurrCurve] do begin
    { Write current ItemIndex as CurrData of CurrCurve in record }
    CurrData := EvalRadioGroupData.ItemIndex;
    { Enable/Disable SpinEdits depending on chosen Items of RadioGroup }
    if ((CurrData = 0) or (CurrData = 3) or (CurrData = 6)) then begin
      EvalSpinEditLayer.Enabled := True;
      EvalSpinEditLayer.Color := clWindow;
      EvalSpinEditNeuron.Enabled := True;
      EvalSpinEditNeuron.Color := clWindow;
      EvalSpinEditWeight.Enabled := True;
      EvalSpinEditWeight.Color := clWindow;
      EvalSpinEditCycle.Enabled := True;
      EvalSpinEditCycle.Color := clWindow;
      EvalGroupBoxDataProc.Enabled := True;
    end;
    if ((CurrData = 1) or (CurrData = 4) or (CurrData = 7) or
      (CurrData = 2) or (CurrData = 5) or (CurrData = 8) or
      (CurrData = 9) or (CurrData = 10) or (CurrData = 11) or
      (CurrData = 12) or (CurrData = 13) or (CurrData = 14) or
      (CurrData = 15) or (CurrData = 16) or (CurrData = 17)) then begin
      EvalSpinEditLayer.Enabled := True;
      EvalSpinEditLayer.Color := clWindow;
      EvalSpinEditNeuron.Enabled := True;
      EvalSpinEditNeuron.Color := clWindow;
      EvalSpinEditWeight.Enabled := False;
      EvalSpinEditWeight.Color := clBtnFace;
      EvalSpinEditCycle.Enabled := True;
      EvalSpinEditCycle.Color := clWindow;
      EvalGroupBoxDataProc.Enabled := True;
    end;
    if ((CurrData = 18) or (CurrData = 19)) then begin
      EvalSpinEditLayer.Enabled := False;
      EvalSpinEditLayer.Color := clBtnFace;
      EvalSpinEditNeuron.Enabled := False;
      EvalSpinEditNeuron.Color := clBtnFace;
      EvalSpinEditWeight.Enabled := False;
      EvalSpinEditWeight.Color := clBtnFace;
      EvalSpinEditCycle.Enabled := True;
      EvalSpinEditCycle.Color := clWindow;
      EvalGroupBoxDataProc.Enabled := True;
    end;
    if ((CurrData = 20)) then begin
      EvalSpinEditLayer.Enabled := False;
      EvalSpinEditLayer.Color := clBtnFace;
      EvalSpinEditNeuron.Enabled := False;
      EvalSpinEditNeuron.Color := clBtnFace;
      EvalSpinEditWeight.Enabled := False;
      EvalSpinEditWeight.Color := clBtnFace;
      EvalSpinEditCycle.Enabled := False;
      EvalSpinEditCycle.Color := clBtnFace;
      EvalGroupBoxDataProc.Enabled := False;
    end;
  end;
  { Call Procedure to set data for diagram and single-value-panel }
  if (EvalLockUpChange = False) then EvalSetDataDiag;
end;

procedure TBackPropForm.EvalSetDataDiag;
var
  EvalDataList: TRealArray;
  EvalDataListTemp: TRealArray;
begin
  with EvalCurve[EvalCurrCurve] do
    with EvalRadioGroupData do begin
      { Read chosen data for each LearnCycle and write them in local list }
      EvalDataList := TRealArray.Create(0);
      EvalGetDataList(CurrData, EvalDataList);
      if (not (EvalDataList.Count < 1) and not (Items[CurrData] = 'Keine')) then begin
        { Apply chosen data processing }
        case Proc.ProcType of
          { Proc off }
          0: case EvalCurrCurve of
            1: EvalCurveLabel1.Caption := Items[CurrData] + ', nicht bearbeitet';
            2: EvalCurveLabel2.Caption := Items[CurrData] + ', nicht bearbeitet';
            3: EvalCurveLabel3.Caption := Items[CurrData] + ', nicht bearbeitet';
            4: EvalCurveLabel4.Caption := Items[CurrData] + ', nicht bearbeitet';
          end;
          { Correlation }
          1: begin
            { Create second, temporary list to correlate with }
            EvalDataListTemp := TRealArray.Create(0);
            EvalGetDataList(Proc.CorrWith, EvalDataListTemp);
            { Set correlation depth to count of smaller list }
            if (EvalDataList.Count < Proc.CorrDepth) then
              Proc.CorrDepth := (EvalDataList.Count);
            if (EvalDataListTemp.Count < Proc.CorrDepth) then
              Proc.CorrDepth := (EvalDataListTemp.Count);
            { Call procedure for data processing }
            EvalProcCorr(EvalDataList, EvalDataListTemp, Proc.CorrDepth);
            { Free temporary list }
            EvalDataListTemp.Free;
            { Write labels }
            case EvalCurrCurve of
              1: EvalCurveLabel1.Caption := Items[CurrData] + ' - ' +
                Proc.CorrName + ' mit: ' + Items[Proc.CorrWith];
              2: EvalCurveLabel2.Caption := Items[CurrData] +
                ' - ' + Proc.CorrName + ' mit: ' + Items[Proc.CorrWith];
              3: EvalCurveLabel3.Caption := Items[CurrData] +
                ' - ' + Proc.CorrName + ' mit: ' + Items[Proc.CorrWith];
              4: EvalCurveLabel4.Caption := Items[CurrData] +
                ' - ' + Proc.CorrName + ' mit: ' + Items[Proc.CorrWith];
            end;
          end;
          { Statistics }
          2: begin
            { Call procedure for data processing }
            EvalProcStat(EvalDataList, Proc.StatMoment, Proc.StatAdaptConst);
            { Write labels }
            case EvalCurrCurve of
              1: EvalCurveLabel1.Caption := Items[CurrData] +
                ': ' + Proc.StatMomentName;
              2: EvalCurveLabel2.Caption := Items[CurrData] +
                ': ' + Proc.StatMomentName;
              3: EvalCurveLabel3.Caption := Items[CurrData] +
                ': ' + Proc.StatMomentName;
              4: EvalCurveLabel4.Caption := Items[CurrData] +
                ': ' + Proc.StatMomentName;
            end;
          end;
          { Filter }
          3: begin
            { Call procedure for data processing }
            EvalProcFilt(EvalDataList, Proc.FiltConst);
            { Write labels }
            case EvalCurrCurve of
              1: EvalCurveLabel1.Caption := Items[CurrData] +
                ' gefilt. mit: ' + Proc.FiltName;
              2: EvalCurveLabel2.Caption := Items[CurrData] +
                ' gefilt. mit: ' + Proc.FiltName;
              3: EvalCurveLabel3.Caption := Items[CurrData] +
                ' gefilt. mit: ' + Proc.FiltName;
              4: EvalCurveLabel4.Caption := Items[CurrData] +
                ' gefilt. mit: ' + Proc.FiltName;
            end;
          end;
          { Operations }
          4: begin
            { Create second, temporary list to operate with }
            EvalDataListTemp := TRealArray.Create(0);
            EvalGetDataList(Proc.OperWith, EvalDataListTemp);
            { Call procedure for data processing }
            EvalProcOper(EvalDataList, EvalDataListTemp, Proc.OperKind, Proc.OperAdaptConst);
            { Free temporary list }
            EvalDataListTemp.Free;
            { Write labels }
            case EvalCurrCurve of
              1: EvalCurveLabel1.Caption := Items[CurrData] +
                ' ' + Proc.OperName + ' mit: ' + Items[Proc.OperWith];
              2: EvalCurveLabel2.Caption := Items[CurrData] +
                ' ' + Proc.OperName + ' mit: ' + Items[Proc.OperWith];
              3: EvalCurveLabel3.Caption := Items[CurrData] +
                ' ' + Proc.OperName + ' mit: ' + Items[Proc.OperWith];
              4: EvalCurveLabel4.Caption := Items[CurrData] +
                ' ' + Proc.OperName + ' mit: ' + Items[Proc.OperWith];
            end;
          end;
          5: begin
          end;
          6: begin
          end;
          7: begin
          end;
          8: begin
          end;
        end;
      end
      else case EvalCurrCurve of
        1: EvalCurveLabel1.Caption := 'Leer';
        2: EvalCurveLabel2.Caption := 'Leer';
        3: EvalCurveLabel3.Caption := 'Leer';
        4: EvalCurveLabel4.Caption := 'Leer';
      end;
    end;
  { Print curve in diagram }
  with BackPropEvalDiagForm do begin
    { Curve 1 }
    if (EvalCurrCurve = 1) then EvalDiagSetDataList1(EvalDataList);
    { Curve 2 }
    if (EvalCurrCurve = 2) then EvalDiagSetDataList2(EvalDataList);
    { Curve 3 }
    if (EvalCurrCurve = 3) then EvalDiagSetDataList3(EvalDataList);
    { Curve 4 }
    if (EvalCurrCurve = 4) then EvalDiagSetDataList4(EvalDataList);
  end;
  { Free EvalDataList }
  EvalDataList.Free;
  { Write value in PanelSingleValue }
  EvalSetDataSingle;
end;

procedure TBackPropForm.EvalSetDataSingle;
var
  EvalDataList: TRealArray;
  HelpValue: Double;
  HelpString: string;
begin
  { Read chosen data for each LearnCycle and write them in local list }
  EvalDataList := TRealArray.Create(0);
  with EvalCurve[EvalCurrCurve] do begin
    EvalGetDataList(CurrData, EvalDataList);
    if (not (EvalDataList.Count < 1) and not
      (EvalRadioGroupData.Items[CurrData] = 'Keine')) then begin
      { Write value in PanelSingleValue }
      HelpValue := EvalDataList[Cycle - 1];
      if (HelpValue >= 1000) then Str(HelpValue : 8 : 0, HelpString)
      else Str(HelpValue : 3 : 5, HelpString);
      EvalPanelSingleValue.Caption := HelpString;
    end
    else EvalPanelSingleValue.Caption := '';
  end;
  { Free EvalDataList }
  EvalDataList.Free;
end;

procedure TBackPropForm.EvalSpeedButtonShowDiagClick(Sender: TObject);
begin
  BackPropEvalDiagForm.Visible := EvalSpeedButtonShowDiag.Down;
  EvalBitBtnPrint.Enabled := EvalSpeedButtonShowDiag.Down;
  EvalBitBtnPrintAll.Enabled := EvalSpeedButtonShowDiag.Down;
end;

procedure TBackPropForm.EvalSpinEditCycleChange(Sender: TObject);
var
  HelpValue: Integer;
begin
  with EvalSpinEditCycle do try
    HelpValue := Value;
    { Check the given value for conflict with net component }
    if ((HelpValue < 1) or (HelpValue > BackPropBackPropNet.Cycles)) then begin
      MessageBeep(0);
      MessageDlg('Ergebnis existiert nicht. Wert fr Lernschritt prfen!' + #13#10 +
        '(mindestens 1, hchstens maximale Lernschrittanzahl)', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end
    else begin
      EvalCurve[EvalCurrCurve].Cycle := HelpValue;
      EvalSetDataSingle;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Lernschrittanzahl muss Integer sein!', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.EvalSpinEditLayerChange(Sender: TObject);
var
  HelpValue: Integer;
begin
  with EvalSpinEditLayer do try
    HelpValue := Value;
    { Check the given value for conflict with net component }
    if ((HelpValue < 2) or (HelpValue > (BackPropBackPropNet.LayerNo + 1))) then begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht zeichnen. Wert fr Schicht prfen! ' + #13#10 +
        '(mindestens 2, hchstens Schichtanzahl im Netz)', mtError, [mbOk], 0);
      Value := 2;
      SetFocus;
    end
    else begin
      { Write layer value in record of current curve }
      EvalCurve[EvalCurrCurve].Layer := Value;
      with EvalSpinEditNeuron do begin
        { Change number of neuron depending on chosen layer }
        MaxValue := BackPropBackPropNet.Items[EvalSpinEditLayer.Value - 2].NeuronNo;
        if (MaxValue) > 1 then MinValue := 1 else MinValue := 0;
        Value := 1;
      end;
      EvalSpinEditNeuronChange(Self);
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Schicht muss Integer sein!', mtError, [mbOk], 0);
      Value := 2;
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.EvalSpinEditNeuronChange(Sender: TObject);
var
  HelpValue: Integer;
begin
  with EvalSpinEditNeuron do try
    HelpValue := Value;
    { Check the given value for conflict with net component }
    if (HelpValue < 1) or (HelpValue > BackPropBackPropNet.
      Items[EvalSpinEditLayer.Value - 2].NeuronNo) then begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht zeichnen. Wert fr Neuron prfen! ' + #13#10 +
      '(mindestens 1, hchstens Neuronanzahl in Schicht)', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end
    else begin
      { Write neuron value in record of current curve }
      EvalCurve[EvalCurrCurve].Neuron := Value;
      with EvalSpinEditWeight do begin
        { Change number of weight depending on chosen neuron }
        MaxValue := BackPropBackPropNet.Items[EvalSpinEditLayer.Value - 2].
          Items[EvalSpinEditNeuron.Value - 1].Weights.Count;
        if (MaxValue > 1) then MinValue := 1 else MinValue := 0;
        Value := 1;
        EvalSpinEditWeightChange(Self);
      end;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Neuron muss Integer sein!', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.EvalSpinEditWeightChange(Sender: TObject);
var
  HelpValue: Integer;
begin
  with EvalSpinEditWeight do try
    HelpValue := Value;
    { Check the given value for conflict with net component }
    if (HelpValue < 1) or (HelpValue > BackPropBackPropNet.Items[EvalSpinEditLayer.Value - 2].
      Items[EvalSpinEditNeuron.Value - 1].Weights.Count) then begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht zeichnen. Wert fr Gewicht prfen! ' + #13#10 +
        '(mindestens 1, hchstens Gewichtanzahl des Neurons)', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end
    else begin
      { Write weight value in record of current curve }
      EvalCurve[EvalCurrCurve].Weight := Value;
      if (EvalLockUpChange = False) then EvalSetDataDiag;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Gewicht muss Integer sein!', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  end;
end;

{ Section 'Form' following }

procedure TBackPropForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if (MessageDlg('Formular "BackPropagation" wirklich schliessen? ' +
    'Alle Daten gehen verloren, insofern nicht vorher in Dateien gespeichert!',
    mtConfirmation, [mbYes, mbNo], 0) = mrYes) then CanClose := True
  else CanClose := False;
end;

procedure TBackPropForm.FormCreate(Sender: TObject);
var
  ExeFilePath: string;
begin
  { Check for authoring identification - copyright }
  if (BackPropLabelAuthor.Caption <> 'ch') then FormDestroy(Self);
  { Create BackPropEvalDiagForm }
  try
    BackPropEvalDiagForm := TBackPropEvalDiagForm.Create(Self);
    BackPropEvalDiagForm.Hide;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg(E.ClassName + '(Fehler beim Erstellen des Diagramm-Formulars)!', mtWarning, [mbOk], 0);
    end;
  end;
  with Application do begin
    { Get path of Application }
    ExeFilePath := ExtractFilePath(ExeName);
    { Write hints in StatusBar }
    OnHint := BackPropStatusBarShowHint;
    { Set HelpFile }
    HelpFile := ExeFilePath + 'Help\BackProp.hlp';
  end;
  { Load WaitCursor }
  Screen.Cursors[crBackPropWaitCursor] := LoadCursor(HInstance, 'Wait');
  { Set status }
  BackPropSetStatus(0, 'Warte auf Eingabe');
  BackPropStatusBar.Refresh;
  { Choose page 'Config' as first active page }
  BackPropPageControl.ActivePage := ConfigTabSheet;
  BackPropPageControlChange(Self);
  { Read Configuration from component }
  ConfigBitBtnNewInitClick(Self);
  { Pattern }
  PattBitBtnNewInitClick(Self);
  { Read LearnParameters from net component; write them on form }
  LearnReadParams;
  { Write values into ComboBoxRecallVectInp }
  with RecallComboBoxRecallVectInp do begin
    Clear;
    Items.Add(IntToStr(Items.Count + 1));
    ItemIndex := (Items.Count - 1);
  end;
  { Disable ComboBoxRecallVectOutp }
  with RecallComboBoxRecallVectOutp do begin
    Clear;
    Enabled := False;
    Color := clBtnFace;
  end;
  RecallListBoxOutp.Clear;
  RecallEditRecallSetDimInp.Text := IntToStr(BackPropBackPropNet.RecallSet.InputNo);
  RecallPanelRecallSetDimOutp.Caption := '';
  { Setting panel showing total number of recall vectors }
  with RecallComboBoxRecallVectInp do
    RecallPanelNrRecallVect.Caption := IntToStr(Items.Count);
  { Call procedure to set ListBoxes }
  RecallComboBoxRecallVectInpChange(Self);
  { Write file name in Edit }
  RecallEditFileRcl.Text := 'Untitled.rcl';
end;

procedure TBackPropForm.FormDestroy(Sender: TObject);
begin
  { Free all public lists of BackPropEvalDiagForm }
  with BackPropEvalDiagForm do begin
    EvalDiagDataList1.Free;
    EvalDiagDataList2.Free;
    EvalDiagDataList3.Free;
    EvalDiagDataList4.Free;
    Free;
  end;
  { Free all existing lists that have stored data of previous learning }
  LearnFreeAllLearnLists;
  { Parameters for diagrams }
  LearnErrorList.Free;
  LearnWeightsList.Free;
  { Free all existing lists having stored data of pruning sessions }
  PrunFreeAllPrunLists;
end;

{ TabSheet 'Learn' following }

procedure TBackPropForm.LearnBitBtnBreakClick(Sender: TObject);
begin
  { Interrupt learning process }
  BackPropBackPropNet.Break := True;
end;

procedure TBackPropForm.LearnBitBtnNewInitClick(Sender: TObject);
begin
  with BackPropBackPropNet do begin
    { Reset all learn parameters to random values and disable TabSheet Eval }
    LearnBitBtnResetClick(Self);
    { Write initial values into net component }
    AdaptConst := 1;
    AdaptConstFactor := 1; // AdaptConstStep := 0, respectively
    CycleLimit := 640;
    ErrorLimit := 0.01;
    Interval := 1;
    LearnMethod := lmEpochal;
    LearnParams := [lpWeights];
    LearnRate := 0.7;
    LearnRateDec := 1.0;
    LearnRateInc := 1.0;
    Momentum := 0.3;
    ParamsMax := 0.9;
    ParamsMin := -0.9;
  end;
  { Read values out of net component; write them into DialogComponents }
  LearnReadParams;
end;

procedure TBackPropForm.LearnBitBtnOpenClick(Sender: TObject);
begin
  with BackPropNetFileDialog do try
    DialogMode := dmOpen;
    Title := 'ffnen einer LernDatenDatei';
    { Filter on Learn }
    FilterIndex := 3;
    FileName := LearnEditFileLrn.Text;
    DefaultExt := 'lrn';
    if (Execute = True) then begin
      BackPropBackPropNet.LoadFromFile(FileName);
      { Write FileName in LearnEditFileLrn }
      LearnEditFileLrn.Text := ExtractFileName(FileName);
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TBackPropForm.LearnBitBtnResetClick(Sender: TObject);
begin
  { Reset all learn parameters to random values }
  BackPropBackPropNet.ResetAll;
  { Disable elements on TabSheet Evaluation }
  EvalDisableAll;
end;

procedure TBackPropForm.LearnBitBtnStartClick(Sender: TObject);
begin
  try
    { Set WaitCursor }
    Screen.Cursor := crBackPropWaitCursor;
    { Set status }
    BackPropSetStatus(1, 'bergebe Parameter');
    BackPropStatusBar.Refresh;
    { Set procedures for events within net component }
    with BackPropBackPropNet do begin
      OnExit := BackPropBackPropNetLearnExit;
      OnInterval := BackPropBackPropNetLearnInterval;
      { Set interval in general to 1 }
      Interval := 1;
    end;
    if (LearnCheckBoxCollectResults.Checked = True) then begin
      { Set status }
      BackPropSetStatus(1, 'Lsche alte Listen; kann lange dauern');
      BackPropStatusBar.Refresh;
      { Set interval to hand over the resulting values during learn process }
      with BackPropBackPropNet do
        case LearnComboBoxCollectResults.ItemIndex of
          0: Interval := 1;
          1: Interval := 5;
          2: Interval := 10;
          3: Interval := 50;
          4: Interval := 100;
          5: Interval := 500;
          6: Interval := 1000;
          7: Interval := 5000;
          8: Interval := 10000;
          9: Interval := 50000;
          10: Interval := 100000;
        end;
      { Free all existing lists that have stored data of previous learning }
      LearnFreeAllLearnLists;
      { Create global net lists to collect values of learn parameters }
      { Weight, bias, slope }
      LearnNetListNeuronWeights := TList.Create;
      LearnNetListNeuronBias := TList.Create;
      LearnNetListNeuronSlope := TList.Create;
      { WeightDelta, BiasDelta, SlopeDelta }
      LearnNetListNeuronWeightsDelta := TList.Create;
      LearnNetListNeuronBiasDelta := TList.Create;
      LearnNetListNeuronSlopeDelta := TList.Create;
      { AdaptWeights, AdaptBias, AdaptSlope }
      LearnNetListNeuronAdaptWeights := TList.Create;
      LearnNetListNeuronAdaptBias := TList.Create;
      LearnNetListNeuronAdaptSlope := TList.Create;
      { Typical values of neuron for just one pattern }
      LearnNetListNeuronNetInput := TList.Create;
      LearnNetListNeuronActivity := TList.Create;
      LearnNetListNeuronOutput := TList.Create;
      LearnNetListNeuronOutputDeriv := TList.Create;
      LearnNetListNeuronDelta := TList.Create;
      LearnNetListNeuronError := TList.Create;
      { Other parameters (Mean value over the whole patternSet) }
      LearnNetListNeuronActivityAll := TList.Create;
      LearnNetListNeuronPowerAll := TList.Create;
      LearnNetListNeuronOutputAll := TList.Create;
      { Global parameters of net }
      LearnNetListNetGlobalError := TRealArray.Create(0);
      LearnNetListNetLearnRateAdapt := TRealArray.Create(0);
    end;
    { Set status }
    BackPropSetStatus(2, 'Lerne Netz an');
    BackPropStatusBar.Refresh;
    { Start learning process of the net }
    BackPropBackPropNet.Learn;
    { Save data in file }
    if (LearnCheckBoxSave.Checked = True) then
      with BackPropNetFileDialog do try
        DialogMode := dmSave;
        Title := 'Speichern einer LernDatenDatei';
        { Filter on Learn }
        FilterIndex := 3;
        FileName := LearnEditFileLrn.Text;
        DefaultExt := 'lrn';
        if Execute then begin
          BackPropBackPropNet.SaveToFile(FileName);
          LearnEditFileLrn.Text := ExtractFileName(FileName);
        end;
      except
        on E: Exception do begin
          MessageBeep(0);
          MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
        end;
      end;
    { Enable/Disable elements }
    EvalDisableAll;
    if (LearnCheckBoxCollectResults.Checked = True) then begin
      with BackPropBackPropNet do
        if (Cycles < Interval) then begin
          MessageBeep(0);
          MessageDlg('Lerndaten konnten nicht in Arrays erfasst werden.',
            mtInformation, [mbOk], 0);
        end
        else begin
          EvalBitBtnNewInitClick(Self);
          EvalEnableAll;
        end;
    end;
  finally
    { Set status }
    BackPropSetStatus(0, 'Warte auf Eingabe');
    BackPropStatusBar.Refresh;
    { Reset WaitCursor }
    Screen.Cursor := crDefault;
  end;
end;

procedure TBackPropForm.LearnBitBtnWeightChoiceClick(Sender: TObject);
begin
  { Fill elements of weight choice ComboBox }
  with LearnSpinEditWeightChoiceLayer do begin
    MaxValue := (BackPropBackPropNet.LayerNo + 1);
    if (MaxValue > 2) then MinValue := 2 else MinValue := 1;
    Value := 2;
  end;
  LearnSpinEditWeightChoiceLayerChange(Self);
end;

procedure TBackPropForm.LearnCheckBoxAdaptiveLearningClick(Sender: TObject);
var
  HelpValueReciprocal: Double;
begin
  with BackPropBackPropNet do
    if (LearnCheckBoxAdaptiveLearning.Checked = True) then begin
      if (Sender <> Self) then begin
        AdaptConst := 8;
        AdaptConstFactor := 1;
        AdaptConstStep := 0;
      end;
      LearnEditAdaptConst.Text := IntToStr(AdaptConst);
      try
        HelpValueReciprocal := (1 / AdaptConst);
      except
        on EDivByZero do HelpValueReciprocal := 0;
      end;
      LearnPanelAdaptConstReciprocal.Caption := FloatToStr(HelpValueReciprocal);
      LearnEditAdaptConstFactor.Text := FloatToStr(AdaptConstFactor);
      LearnEditAdaptConstStep.Text := FloatToStr(AdaptConstStep);
      LearnEditAdaptConst.Color := clWindow;
      LearnEditAdaptConstFactor.Color := clWindow;
      LearnEditAdaptConstStep.Color := clWindow;
      LearnGroupBoxAdaptiveLearning.Enabled := True;
    end
    else begin
      AdaptConst := 0;
      AdaptConstFactor := 1;
      AdaptConstStep := 0;
      LearnEditAdaptConst.Text := IntToStr(AdaptConst);
      LearnPanelAdaptConstReciprocal.Caption := '';
      LearnEditAdaptConstFactor.Text := FloatToStr(AdaptConstFactor);
      LearnEditAdaptConstStep.Text := FloatToStr(AdaptConstStep);
      LearnEditAdaptConst.Color := clBtnFace;
      LearnEditAdaptConstFactor.Color := clBtnFace;
      LearnEditAdaptConstStep.Color := clBtnFace;
      LearnGroupBoxAdaptiveLearning.Enabled := False;
    end;
end;

procedure TBackPropForm.LearnCheckBoxCollectResultsClick(Sender: TObject);
begin
  if (LearnCheckBoxCollectResults.Checked = True) then begin
    MessageBeep(0);
    MessageDlg('Groe Intervalle whlen, um Kapazittsprobleme zu vermeiden!',
      mtWarning, [mbOk], 0);
  end;
end;

procedure TBackPropForm.LearnCheckBoxLearnRateRecursiveClick(Sender: TObject);
begin
  with BackPropBackPropNet do
    if (LearnCheckBoxLearnRateRecursive.Checked = True) then begin
      if (Sender <> Self) then begin
        LearnRateInc := 1.04;
        LearnRateDec := 0.7;
      end;
      LearnEditLearnRateInc.Text := FloatToStr(LearnRateInc);
      LearnEditLearnRateDec.Text := FloatToStr(LearnRateDec);
      LearnEditLearnRateInc.Color := clWindow;
      LearnEditLearnRateDec.Color := clWindow;
      LearnGroupBoxLearnRateRecursive.Enabled := True;
    end
    else begin
      LearnRateInc := 1.0;
      LearnRateDec := 1.0;
      LearnEditLearnRateInc.Text := FloatToStr(LearnRateInc);
      LearnEditLearnRateDec.Text := FloatToStr(LearnRateDec);
      LearnEditLearnRateInc.Color := clBtnFace;
      LearnEditLearnRateDec.Color := clBtnFace;
      LearnGroupBoxLearnRateRecursive.Enabled := False;
    end;
end;

procedure TBackPropForm.LearnCheckBoxLearnSlopeClick(Sender: TObject);
begin
  with BackPropBackPropNet do begin
    if (LearnCheckBoxLearnSlope.Checked = True) then LearnParams := (LearnParams + [lpSlope])
    else LearnParams := (LearnParams - [lpSlope]);
    if (ConfigListBoxLay.ItemIndex > 0) then
      ConfigEditFunctOutpSlope.Text := FloatToStr(Items[(ConfigListBoxLay.ItemIndex - 1)].Slope);
  end;
end;

procedure TBackPropForm.LearnCheckBoxLearnThreshClick(Sender: TObject);
begin
  with BackPropBackPropNet do begin
    if (LearnCheckBoxLearnThresh.Checked = True) then LearnParams := (LearnParams + [lpBias])
    else LearnParams := (LearnParams - [lpBias]);
    if (ConfigListBoxLay.ItemIndex > 0) then
      ConfigEditFunctOutpThresh.Text := FloatToStr(Items[(ConfigListBoxLay.ItemIndex - 1)].Bias);
  end;
end;

procedure TBackPropForm.LearnCheckBoxLearnWeightsClick(Sender: TObject);
begin
  with BackPropBackPropNet do
{    if (LearnCheckBoxLearnWeights.Checked = True) then
      LearnParams := (LearnParams + [lpWeights])
    else LearnParams := (LearnParams - [lpWeights]);}
    if (LearnCheckBoxLearnWeights.Checked = False) then begin
      MessageBeep(0);
      MessageDlg('Gewichte mssen immer gelernt werden!', mtWarning, [mbOk], 0);
      LearnCheckBoxLearnWeights.Checked := True;
    end;
end;

procedure TBackPropForm.LearnCollectData;
var
  { Counting variables representing number of Layer, Neuron, Weight }
  k: Integer;
  i: Integer;
  j: Integer;
  { Weight, bias, slope }
  LearnLayerListNeuronWeights: TList;
  LearnLayerListNeuronBias: TList;
  LearnLayerListNeuronSlope: TList;
  { WeightDelta, BiasDelta, SlopeDelta }
  LearnLayerListNeuronWeightsDelta: TList;
  LearnLayerListNeuronBiasDelta: TList;
  LearnLayerListNeuronSlopeDelta: TList;
  { AdaptWeights, AdaptBias, AdaptSlope }
  LearnLayerListNeuronAdaptWeights: TList;
  LearnLayerListNeuronAdaptBias: TList;
  LearnLayerListNeuronAdaptSlope: TList;
  { Typical values of neuron for just one pattern }
  LearnLayerListNeuronNetInput: TList;
  LearnLayerListNeuronActivity: TList;
  LearnLayerListNeuronOutput: TList;
  LearnLayerListNeuronOutputDeriv: TList;
  LearnLayerListNeuronDelta: TList;
  LearnLayerListNeuronError: TList;
  { Other parameters (Mean value over the whole patternSet) }
  LearnLayerListNeuronActivityAll: TList;
  LearnLayerListNeuronPowerAll: TList;
  LearnLayerListNeuronOutputAll: TList;
  { Weight, bias, slope }
  LearnNeuronListNeuronWeights: TList;
  LearnNeuronListNeuronBias: TRealArray;
  LearnNeuronListNeuronSlope: TRealArray;
  { WeightDelta, BiasDelta, SlopeDelta }
  LearnNeuronListNeuronWeightsDelta: TList;
  LearnNeuronListNeuronBiasDelta: TRealArray;
  LearnNeuronListNeuronSlopeDelta: TRealArray;
  { AdaptWeights, AdaptBias, AdaptSlope }
  LearnNeuronListNeuronAdaptWeights: TList;
  LearnNeuronListNeuronAdaptBias: TRealArray;
  LearnNeuronListNeuronAdaptSlope: TRealArray;
  { Typical values of neuron for just one pattern }
  LearnNeuronListNeuronNetInput: TRealArray;
  LearnNeuronListNeuronActivity: TRealArray;
  LearnNeuronListNeuronOutput: TRealArray;
  LearnNeuronListNeuronOutputDeriv: TRealArray;
  LearnNeuronListNeuronDelta: TRealArray;
  LearnNeuronListNeuronError: TRealArray;
  { Other parameters (Mean value over the whole patternSet) }
  LearnNeuronListNeuronActivityAll: TRealArray;
  LearnNeuronListNeuronPowerAll: TRealArray;
  LearnNeuronListNeuronOutputAll: TRealArray;
  { Neuron weights }
  LearnWeightListNeuronWeights: TRealArray;
  LearnWeightListNeuronWeightsDelta: TRealArray;
  LearnWeightListNeuronAdaptWeights: TRealArray;
begin
  { Create lists for layer parameters }
  { Weight, bias, slope }
  LearnLayerListNeuronWeights := TList.Create;
  LearnLayerListNeuronBias := TList.Create;
  LearnLayerListNeuronSlope := TList.Create;
  { WeightDelta, BiasDelta, SlopeDelta }
  LearnLayerListNeuronWeightsDelta := TList.Create;
  LearnLayerListNeuronBiasDelta := TList.Create;
  LearnLayerListNeuronSlopeDelta := TList.Create;
  { AdaptWeights, AdaptBias, AdaptSlope }
  LearnLayerListNeuronAdaptWeights := TList.Create;
  LearnLayerListNeuronAdaptBias := TList.Create;
  LearnLayerListNeuronAdaptSlope := TList.Create;
  { Typical values of neuron for just one pattern }
  LearnLayerListNeuronNetInput := TList.Create;
  LearnLayerListNeuronActivity := TList.Create;
  LearnLayerListNeuronOutput := TList.Create;
  LearnLayerListNeuronOutputDeriv := TList.Create;
  LearnLayerListNeuronDelta := TList.Create;
  LearnLayerListNeuronError := TList.Create;
  { Other parameters (Mean value over the whole patternSet) }
  LearnLayerListNeuronActivityAll := TList.Create;
  LearnLayerListNeuronPowerAll := TList.Create;
  LearnLayerListNeuronOutputAll := TList.Create;
  for k := 0 to (BackPropBackPropNet.LayerNo - 1) do begin
    { Create lists for neuron parameters }
    { Weight, bias, slope }
    LearnNeuronListNeuronWeights := TList.Create;
    LearnNeuronListNeuronBias := TRealArray.Create(0);
    LearnNeuronListNeuronSlope := TRealArray.Create(0);
    { WeightDelta, BiasDelta, SlopeDelta }
    LearnNeuronListNeuronWeightsDelta := TList.Create;
    LearnNeuronListNeuronBiasDelta := TRealArray.Create(0);
    LearnNeuronListNeuronSlopeDelta := TRealArray.Create(0);
    { AdaptWeights, AdaptBias, AdaptSlope }
    LearnNeuronListNeuronAdaptWeights := TList.Create;
    LearnNeuronListNeuronAdaptBias := TRealArray.Create(0);
    LearnNeuronListNeuronAdaptSlope := TRealArray.Create(0);
    { Typical values of neuron for just one pattern }
    LearnNeuronListNeuronNetInput := TRealArray.Create(0);
    LearnNeuronListNeuronActivity := TRealArray.Create(0);
    LearnNeuronListNeuronOutput := TRealArray.Create(0);
    LearnNeuronListNeuronOutputDeriv := TRealArray.Create(0);
    LearnNeuronListNeuronDelta := TRealArray.Create(0);
    LearnNeuronListNeuronError := TRealArray.Create(0);
    { Other parameters (Mean value over the whole patternSet) }
    LearnNeuronListNeuronActivityAll := TRealArray.Create(0);
    LearnNeuronListNeuronPowerAll := TRealArray.Create(0);
    LearnNeuronListNeuronOutputAll := TRealArray.Create(0);
    { Fill values into lists }
    for i := 0 to (BackPropBackPropNet.Items[k].NeuronNo - 1) do begin
      { Create lists for weights }
      LearnWeightListNeuronWeights := TRealArray.Create(0);
      LearnWeightListNeuronWeightsDelta := TRealArray.Create(0);
      LearnWeightListNeuronAdaptWeights := TRealArray.Create(0);
      { Collect values }
      with BackPropBackPropNet do begin
        { Weights, bias, slope }
        for j := 0 to (Items[k].Items[i].Weights.Count - 1) do
          LearnWeightListNeuronWeights.Add(Items[k].Items[i].Weights[j]);
        LearnNeuronListNeuronWeights.Add(LearnWeightListNeuronWeights);
        LearnNeuronListNeuronBias.Add(Items[k].Items[i].Bias);
        LearnNeuronListNeuronSlope.Add(Items[k].Items[i].Slope);
        { Delta of weights, bias, slope }
        for j := 0 to (Items[k].Items[i].WeightsDelta.Count - 1) do
          LearnWeightListNeuronWeightsDelta.Add(Items[k].Items[i].WeightsDelta[j]);
        LearnNeuronListNeuronWeightsDelta.Add(LearnWeightListNeuronWeightsDelta);
        LearnNeuronListNeuronBiasDelta.Add(Items[k].Items[i].BiasDelta);
        LearnNeuronListNeuronSlopeDelta.Add(Items[k].Items[i].SlopeDelta);
        { Adaptive values of weights, bias, slope }
        for j := 0 to (Items[k].Items[i].AdaptWeights.Count - 1) do
          LearnWeightListNeuronAdaptWeights.Add(Items[k].Items[i].AdaptWeights[j]);
        LearnNeuronListNeuronAdaptWeights.Add(LearnWeightListNeuronAdaptWeights);
        LearnNeuronListNeuronAdaptBias.Add(Items[k].Items[i].AdaptBias);
        LearnNeuronListNeuronAdaptSlope.Add(Items[k].Items[i].AdaptSlope);
        { Typical values of neuron for one pattern }
        LearnNeuronListNeuronNetInput.Add(Items[k].Items[i].NetInput);
        LearnNeuronListNeuronActivity.Add(Items[k].Items[i].Activity);
        LearnNeuronListNeuronOutput.Add(Items[k].Items[i].Output);
        LearnNeuronListNeuronOutputDeriv.Add(Items[k].Items[i].OutputDeriv);
        LearnNeuronListNeuronDelta.Add(Items[k].Items[i].Delta);
        LearnNeuronListNeuronError.Add(Items[k].Items[i].Error);
        { Other parameters (Mean value over the whole patternSet) }
        LearnNeuronListNeuronActivityAll.Add(Items[k].Items[i].ActivityAll);
        LearnNeuronListNeuronPowerAll.Add(Items[k].Items[i].PowerAll);
        LearnNeuronListNeuronOutputAll.Add(Items[k].Items[i].OutputAll);
      end;
    end;
    { Add layer lists to cycle lists }
    { Weight, bias, slope }
    LearnLayerListNeuronWeights.Add(LearnNeuronListNeuronWeights);
    LearnLayerListNeuronBias.Add(LearnNeuronListNeuronBias);
    LearnLayerListNeuronSlope.Add(LearnNeuronListNeuronSlope);
    { WeightDelta, BiasDelta, SlopeDelta }
    LearnLayerListNeuronWeightsDelta.Add(LearnNeuronListNeuronWeightsDelta);
    LearnLayerListNeuronBiasDelta.Add(LearnNeuronListNeuronBiasDelta);
    LearnLayerListNeuronSlopeDelta.Add(LearnNeuronListNeuronSlopeDelta);
    { AdaptWeights, AdaptBias, AdaptSlope }
    LearnLayerListNeuronAdaptWeights.Add(LearnNeuronListNeuronAdaptWeights);
    LearnLayerListNeuronAdaptBias.Add(LearnNeuronListNeuronAdaptBias);
    LearnLayerListNeuronAdaptSlope.Add(LearnNeuronListNeuronAdaptSlope);
    { Typical values of neuron for just one pattern }
    LearnLayerListNeuronNetInput.Add(LearnNeuronListNeuronNetInput);
    LearnLayerListNeuronActivity.Add(LearnNeuronListNeuronActivity);
    LearnLayerListNeuronOutput.Add(LearnNeuronListNeuronOutput);
    LearnLayerListNeuronOutputDeriv.Add(LearnNeuronListNeuronOutputDeriv);
    LearnLayerListNeuronDelta.Add(LearnNeuronListNeuronDelta);
    LearnLayerListNeuronError.Add(LearnNeuronListNeuronError);
    { Other parameters (Mean value over the whole patternSet) }
    LearnLayerListNeuronActivityAll.Add(LearnNeuronListNeuronActivityAll);
    LearnLayerListNeuronPowerAll.Add(LearnNeuronListNeuronPowerAll);
    LearnLayerListNeuronOutputAll.Add(LearnNeuronListNeuronOutputAll);
  end;
  { Add all lists for current cycle to global LearnLists }
  { Weight, bias, slope }
  LearnNetListNeuronWeights.Add(LearnLayerListNeuronWeights);
  LearnNetListNeuronBias.Add(LearnLayerListNeuronBias);
  LearnNetListNeuronSlope.Add(LearnLayerListNeuronSlope);
  { WeightDelta, BiasDelta, SlopeDelta }
  LearnNetListNeuronWeightsDelta.Add(LearnLayerListNeuronWeightsDelta);
  LearnNetListNeuronBiasDelta.Add(LearnLayerListNeuronBiasDelta);
  LearnNetListNeuronSlopeDelta.Add(LearnLayerListNeuronSlopeDelta);
  { AdaptWeights, AdaptBias, AdaptSlope }
  LearnNetListNeuronAdaptWeights.Add(LearnLayerListNeuronAdaptWeights);
  LearnNetListNeuronAdaptBias.Add(LearnLayerListNeuronAdaptBias);
  LearnNetListNeuronAdaptSlope.Add(LearnLayerListNeuronAdaptSlope);
  { Typical values of neuron for just one pattern }
  LearnNetListNeuronNetInput.Add(LearnLayerListNeuronNetInput);
  LearnNetListNeuronActivity.Add(LearnLayerListNeuronActivity);
  LearnNetListNeuronOutput.Add(LearnLayerListNeuronOutput);
  LearnNetListNeuronOutputDeriv.Add(LearnLayerListNeuronOutputDeriv);
  LearnNetListNeuronDelta.Add(LearnLayerListNeuronDelta);
  LearnNetListNeuronError.Add(LearnLayerListNeuronError);
  { Other parameters (Mean value over the whole patternSet) }
  LearnNetListNeuronActivityAll.Add(LearnLayerListNeuronActivityAll);
  LearnNetListNeuronPowerAll.Add(LearnLayerListNeuronPowerAll);
  LearnNetListNeuronOutputAll.Add(LearnLayerListNeuronOutputAll);
  { Global parameters of net }
  with BackPropBackPropNet do begin
    LearnNetListNetGlobalError.Add(GlobalError);
    LearnNetListNetLearnRateAdapt.Add(LearnRateAdapt);
  end;
end;

procedure TBackPropForm.LearnEditAdaptConstExit(Sender: TObject);
var
  HelpValue: Integer;
  HelpValueReciprocal: Double;
begin
  { Check if adaption constant string is value; if yes then hand over to net }
  with LearnEditAdaptConst do try
    HelpValue := StrToInt(Text);
    if (HelpValue <= 0) then begin
      MessageBeep(0);
      MessageDlg('Wert muss grsser als 0 sein!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.AdaptConst);
      SetFocus;
    end
    else begin
      try
        HelpValueReciprocal := (1 / HelpValue);
      except
        on EDivByZero do HelpValueReciprocal := 0;
      end;
      LearnPanelAdaptConstReciprocal.Caption := FloatToStr(HelpValueReciprocal);
      BackPropBackPropNet.AdaptConst := HelpValue;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Adaptionskonstante prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.AdaptConst);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditAdaptConstFactorExit(Sender: TObject);
var
  HelpValue: Integer;
begin
  { Check if adaption constant factor string is value; hand over to net }
  LearnEditAdaptConstStep.Text := '0';
  BackPropBackPropNet.AdaptConstStep := 0;
  with LearnEditAdaptConstFactor do try
    { Check for range }
    HelpValue := StrToInt(Text);
    if (HelpValue <= 0) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Faktor muss grsser null sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.AdaptConstFactor);
      SetFocus;
    end
    else BackPropBackPropNet.AdaptConstFactor := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Faktor prfen (Integerwert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.AdaptConstFactor);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditAdaptConstFactorKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditAdaptConstFactorExit(Self);
end;

procedure TBackPropForm.LearnEditAdaptConstKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditAdaptConstExit(Self);
end;

procedure TBackPropForm.LearnEditAdaptConstStepExit(Sender: TObject);
var
  HelpValue: Integer;
begin
  { Check if adaption constant step string is value; hand over to net }
  LearnEditAdaptConstFactor.Text := '1';
  BackPropBackPropNet.AdaptConstFactor := 1;
  with LearnEditAdaptConstStep do try
    HelpValue := StrToInt(Text);
    if (HelpValue < 0) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Addendum muss grsser oder gleich 0 sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.AdaptConstStep);
      SetFocus;
    end
    else BackPropBackPropNet.AdaptConstStep := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Summanden prfen (Integerwert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.AdaptConstStep);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditAdaptConstStepKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditAdaptConstStepExit(Self);
end;

procedure TBackPropForm.LearnEditCycleLimitExit(Sender: TObject);
var
  HelpValue: Integer;
begin
  { Check if max cycle limit string is value; if yes then hand over to net }
  with LearnEditCycleLimit do try
    HelpValue := StrToInt(Text);
    if (HelpValue <= 0) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Lernschrittanzahl muss grsser null sein!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.CycleLimit);
      SetFocus;
    end
    else BackPropBackPropNet.CycleLimit := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Lernschrittanzahl prfen (Integerwert)!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.CycleLimit);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditCycleLimitKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditCycleLimitExit(Self);
end;

procedure TBackPropForm.LearnEditErrorLimitExit(Sender: TObject);
var
  HelpValue: Double;
begin
  { Check if error limit string is value; if yes then hand over to net }
  with LearnEditErrorLimit do try
    HelpValue := StrToFloat(Text);
    if (HelpValue <= 0) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Fehlergrenze muss > 0 sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.ErrorLimit);
      SetFocus;
    end
    else BackPropBackPropNet.ErrorLimit := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Fehlergrenze prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.ErrorLimit);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditErrorLimitKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditErrorLimitExit(Self);
end;

procedure TBackPropForm.LearnEditLearnRateDecExit(Sender: TObject);
var
  HelpValue: Double;
begin
  { Check if lern rate decrease string is value; if yes then hand over to net }
  with LearnEditLearnRateDec do try
    HelpValue := StrToFloat(Text);
    if (HelpValue > 1.0) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Verringerung muss < 1 sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.LearnRateDec);
      SetFocus;
    end
    else begin
      BackPropBackPropNet.LearnRateDec := HelpValue;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Verringerung prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.LearnRateDec);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditLearnRateDecKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditLearnRateDecExit(Self);
end;

procedure TBackPropForm.LearnEditLearnRateExit(Sender: TObject);
var
  HelpValue: Double;
begin
  { Check if learn rate string is value; if yes then hand over to net }
  with LearnEditLearnRate do try
    HelpValue := StrToFloat(Text);
    { Check if learn rate is placed in defined range }
    if (StrToFloat(LearnEditLearnRate.Text) <= 0) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Lernrate muss > 0 sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.LearnRate);
      SetFocus;
    end
    else BackPropBackPropNet.LearnRate := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Lernrate prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.LearnRate);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditLearnRateIncExit(Sender: TObject);
var
  HelpValue: Double;
begin
  { Check if lern rate increase string is value; if yes then hand over to net }
  with LearnEditLearnRateInc do try
    HelpValue := StrToFloat(Text);
    if (HelpValue < 1.0) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Erhhung muss >= 1 sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.LearnRateInc);
      SetFocus;
    end
    else BackPropBackPropNet.LearnRateInc := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Erhhung prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.LearnRateInc);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditLearnRateIncKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditLearnRateIncExit(Self);
end;

procedure TBackPropForm.LearnEditLearnRateKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditLearnRateExit(Self);
end;

procedure TBackPropForm.LearnEditMomentumExit(Sender: TObject);
var
  HelpValue: Double;
begin
  { Check if momentum string is value; if yes then hand over to net }
  with LearnEditMomentum do try
    HelpValue := StrToFloat(Text);
    { Check if momentum is placed in defined range }
    if ((HelpValue < 0) or (HelpValue > 1)) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Momentum muss zwischen 0 und 1 liegen!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.Momentum);
      SetFocus;
    end
    else BackPropBackPropNet.Momentum := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Momentum prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.Momentum);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditMomentumKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditMomentumExit(Self);
end;

procedure TBackPropForm.LearnEditParamsMaxExit(Sender: TObject);
var
  HelpValue: Double;
begin
  { Check if maximum of initialization string is value; hand over }
  with LearnEditParamsMax do try
    HelpValue := StrToFloat(Text);
    if (HelpValue <= StrToFloat(LearnEditParamsMin.Text)) then begin
      MessageBeep(0);
      MessageDlg('Maximum der Initialisierung muss > Minimum sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.ParamsMax);
      SetFocus;
    end
    else BackPropBackPropNet.ParamsMax := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Obergrenze prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.ParamsMax);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditParamsMaxKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditParamsMaxExit(Self);
end;

procedure TBackPropForm.LearnEditParamsMinExit(Sender: TObject);
var
  HelpValue: Double;
begin
  { Check if minimum of initialization string is value; hand over }
  with LearnEditParamsMin do try
    HelpValue := StrToFloat(Text);
    if (HelpValue >= StrToFloat(LearnEditParamsMax.Text)) then begin
      MessageBeep(0);
      MessageDlg('Minimum der Initialisierung muss < Maximum sein!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.ParamsMin);
      SetFocus;
    end
    else BackPropBackPropNet.ParamsMin := HelpValue;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Untergrenze prfen (Fliesskommawert)!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.ParamsMin);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnEditParamsMinKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then LearnEditParamsMinExit(Self);
end;

procedure TBackPropForm.LearnFreeAllLearnLists;
var
  n: Integer; { Net state of one LearnCycle n }
  k: Integer; { Layer state }
  i: Integer; { Neuron state }
begin
  { Free all existing lists that have been storing data of previous learning }
  { Weight, bias, slope }
  if Assigned(LearnNetListNeuronWeights) then begin
    with LearnNetListNeuronWeights do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count -1) do begin
              if (TList(TList(Items[n]).Items[k]).Count > 0) then begin
                for i := 0 to (TList(TList(Items[n]).Items[k]).Count - 1) do begin
                  TRealArray(TList(TList(Items[n]).Items[k]).Items[i]).Free;
                end;
              end;
              TList(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronWeights := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronBias) then begin
    with LearnNetListNeuronBias do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronBias := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronSlope) then begin
    with LearnNetListNeuronSlope do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronSlope := nil;
    end;
  end;
  { WeightDelta, BiasDelta, SlopeDelta }
  if Assigned(LearnNetListNeuronWeightsDelta) then begin
    with LearnNetListNeuronWeightsDelta do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count -1) do begin
              if (TList(TList(Items[n]).Items[k]).Count > 0) then begin
                for i := 0 to (TList(TList(Items[n]).Items[k]).Count - 1) do begin
                  TRealArray(TList(TList(Items[n]).Items[k]).Items[i]).Free;
                end;
              end;
              TList(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronWeightsDelta := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronBiasDelta) then begin
    with LearnNetListNeuronBiasDelta do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronBiasDelta := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronSlopeDelta) then begin
    with LearnNetListNeuronSlopeDelta do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronSlopeDelta := nil;
    end;
  end;
  { AdaptWeights, AdaptBias, AdaptSlope }
  if Assigned(LearnNetListNeuronAdaptWeights) then begin
    with LearnNetListNeuronAdaptWeights do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count -1) do begin
              if (TList(TList(Items[n]).Items[k]).Count > 0) then begin
                for i := 0 to (TList(TList(Items[n]).Items[k]).Count - 1) do begin
                  TRealArray(TList(TList(Items[n]).Items[k]).Items[i]).Free;
                end;
              end;
              TList(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronAdaptWeights := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronAdaptBias) then begin
    with LearnNetListNeuronAdaptBias do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronAdaptBias := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronAdaptSlope) then begin
    with LearnNetListNeuronAdaptSlope do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronAdaptSlope := nil;
    end;
  end;
  { Typical values of neuron for just one pattern }
  if Assigned(LearnNetListNeuronNetInput) then begin
    with LearnNetListNeuronNetInput do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronNetInput := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronActivity) then begin
    with LearnNetListNeuronActivity do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronActivity := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronOutput) then begin
    with LearnNetListNeuronOutput do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronOutput := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronOutputDeriv) then begin
    with LearnNetListNeuronOutputDeriv do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronOutputDeriv := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronDelta) then begin
    with LearnNetListNeuronDelta do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronDelta := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronError) then begin
    with LearnNetListNeuronError do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronError := nil;
    end;
  end;
  { Other parameters (Mean value over the whole patternSet) }
  if Assigned(LearnNetListNeuronActivityAll) then begin
    with LearnNetListNeuronActivityAll do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronActivityAll := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronPowerAll) then begin
    with LearnNetListNeuronPowerAll do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronPowerAll := nil;
    end;
  end;
  if Assigned(LearnNetListNeuronOutputAll) then begin
    with LearnNetListNeuronOutputAll do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          if (TList(Items[n]).Count > 0) then begin
            for k := 0 to (TList(Items[n]).Count - 1) do begin
              TRealArray(TList(Items[n]).Items[k]).Free;
            end;
          end;
          TList(Items[n]).Free;
        end;
      end;
      Free;
      LearnNetListNeuronOutputAll := nil;
    end;
  end;
  { Global parameters of net }
  if Assigned(LearnNetListNetGlobalError) then begin
    with LearnNetListNetGlobalError do Free;
    LearnNetListNetGlobalError := nil;
  end;
  if Assigned(LearnNetListNetLearnRateAdapt) then begin
    with LearnNetListNetLearnRateAdapt do Free;
    LearnNetListNetLearnRateAdapt := nil;
  end;
end;

procedure TBackPropForm.LearnHideBoxes;
begin
  { Hide all boxes and diagrams }
  LearnRadioGroupMethod.Visible := False;
  LearnGroupBoxLearnProcess.Visible := False;
  LearnGroupBoxLearnParams.Visible := False;
  LearnGroupBoxLimit.Visible := False;
  LearnGroupBoxLearnRateRecursive.Visible := False;
  LearnGroupBoxAdaptiveLearning.Visible := False;
  LearnGroupBoxLearnParamsInit.Visible := False;
  LearnGroupBoxCollectResults.Visible := False;
  { Elements belonging to diagrams }
  LearnGroupBoxWeightChoice.Visible := False;
  LearnPaintBoxBar.Visible := False;
  LearnPaintBoxDiag.Visible := False;
  LearnPaintBoxHinton.Visible := False;
  LearnPanelGlobalError.Visible := False;
  LearnPanelWeights.Visible := False;
end;

procedure TBackPropForm.LearnPaintBoxBarPaint(Sender: TObject);
var
  HelpString: string;
begin
  Invalidate;
  with LearnPaintBoxBar do with Canvas do begin
    { Background }
    Brush.Color := clBtnFace;
    FillRect(Rect(0, 0, Width, Height));
    { Draw 2 lines centred vertical }
    Pen.Width := 2;
    Pen.Style := psSolid;
    Pen.Color := clBlack;
    MoveTo(40, 160); LineTo(210, 160);
    MoveTo(250, 160); LineTo(420, 160);
    { Test if LearnWeightsList exists to prevent AccessViolations }
    if Assigned(LearnWeightsList) then begin
      { Check if list is greater than zero to prevent IndexErrors }
      if (LearnWeightsList.Count > 0) then begin
        { Weight 1 - text }
        with LearnSpinEditWeightChoice1 do begin
          if (LearnWeightsList[Value - 1] > 10) then
            Str(LearnWeightsList[Value - 1] : 4 : 0, HelpString)
          else Str(LearnWeightsList[Value - 1] : 1 : 4, HelpString);
          TextOut(40, 320, IntToStr(Value) + '. Gewicht ' + 'des ' +
            IntToStr(LearnSpinEditWeightChoiceNeuron.Value) + '. Neurons');
          TextOut(40, 335, 'der ' + IntToStr(LearnSpinEditWeightChoiceLayer.
            Value) + '. Schicht = ' + HelpString);
        end;
        { Weight 2 - text}
        with LearnSpinEditWeightChoice2 do begin
          if (LearnWeightsList[Value - 1] > 10) then
            Str(LearnWeightsList[Value - 1] : 4 : 0, HelpString)
          else Str(LearnWeightsList[Value - 1] : 1 : 4, HelpString);
          TextOut(250, 320, IntToStr(Value) + '. Gewicht ' + 'des ' +
            IntToStr(LearnSpinEditWeightChoiceNeuron.Value) + '. Neurons');
          TextOut(250, 335, 'der ' + IntToStr(LearnSpinEditWeightChoiceLayer.
            Value) + '. Schicht = ' + HelpString);
        end;
        { Draw 2 bars sized like chosen weights 1 and 2 of neuron }
        { Bar 1 }
        if (LearnWeightsList[LearnSpinEditWeightChoice1.Value - 1] < 0) then
          Brush.Color := clRed
        else Brush.Color := clGreen;
        FillRect(Rect(60, 160, 190, (160 - Round(LearnWeightsList[
          LearnSpinEditWeightChoice1.Value - 1] * LearnFactYBar))));
        { Bar 2 }
        if (LearnWeightsList[LearnSpinEditWeightChoice2.Value - 1] < 0) then
          Brush.Color := clRed
        else Brush.Color := clGreen;
        FillRect(Rect(270, 160, 400, (160 - Round(LearnWeightsList[
          LearnSpinEditWeightChoice2.Value - 1] * LearnFactYBar))));
      end;
    end;
  end;
end;

procedure TBackPropForm.LearnPaintBoxDiagPaint(Sender: TObject);
var
  i: Integer;
  HelpString: string;
begin
  Invalidate;
  with LearnPaintBoxDiag do with Canvas do begin
    { Background }
    Brush.Color := clBtnFace;
    FillRect(Rect(0, 0, Width, Height));
    { X-axis and arrow }
    Pen.Width := 1;
    Pen.Style := psSolid;
    Pen.Color := clBlack;
    MoveTo(5, (LearnPaintBoxDiag.Height - 30)); LineTo((Width - 10), (Height - 30));
    MoveTo((Width - 10), (Height - 30)); LineTo((Width - 20), (Height - 33));
    MoveTo((Width - 10), (Height - 30)); LineTo((Width - 20), (Height - 27));
    { Y-axis and arrow }
    MoveTo(10, (Height - 25)); LineTo(10, 30);
    MoveTo(10, 30); LineTo(7, 40);
    MoveTo(10, 30); LineTo(13, 40);
    { Grid }
    Pen.Style := psDot;
    for i := 1 to 10 do begin
      MoveTo(10 + (i * Round((Width - 20) / 10)), (LearnPaintBoxDiag.Height - 30));
      LineTo(10 + (i * Round((Width - 20) / 10)), 30);
    end;
    { Display values }
    TextOut((Width - 150), (Height - 25), 'Lernschritte: ' +
      IntToStr(BackPropBackPropNet.Cycles));
    if (LearnErrorMinY > 10) then Str(LearnErrorMinY : 10 : 2, HelpString)
    else Str(LearnErrorMinY : 1 : 8, HelpString);
    TextOut(5, (Height - 25), 'Minimum: ' + HelpString);
    if (LearnErrorMaxY > 10) then Str(LearnErrorMaxY : 10 : 2, HelpString)
    else Str(LearnErrorMaxY : 1 : 8, HelpString);
    TextOut(5, 5, 'Maximum: ' + HelpString);
    { Draw data }
    if (LearnErrorList <> nil) then begin
      if (LearnErrorList.Count > 0) then begin
        Pen.Style := psSolid;
        Pen.Color := clRed;
        MoveTo(10, ((Height - 30) - Round((LearnErrorList[0] - LearnErrorMinY) * LearnFactYDiag)));
        if LearnErrorList.Count > 1 then begin
          for i := 1 to (LearnErrorList.Count - 1) do begin
            LineTo((10 + Round(i * LearnFactXDiag)),
              ((Height - 30) - Round((LearnErrorList[i] - LearnErrorMinY) * LearnFactYDiag)));
          end;
        end;
      end;
    end;
  end;
end;

procedure TBackPropForm.LearnPaintBoxHintonPaint(Sender: TObject);
var
  RectSide: Integer;
  OrigoX1, OrigoY1: Integer;
  OrigoX2, OrigoY2: Integer;
  HelpString: string;
begin
  Invalidate;
  with LearnPaintBoxHinton do with Canvas do begin
    { Background }
    Brush.Color := clBtnFace;
    FillRect(Rect(0, 0, Width, Height));
    { Check if list exists to prevent AccessViolations }
    if Assigned(LearnWeightsList) then begin
      { Check if list contains elements to prevent IndexErrors }
      if (LearnWeightsList.Count > 0) then begin
        { Weight 1 - text }
        with LearnSpinEditWeightChoice1 do begin
          if (LearnWeightsList[Value - 1] > 10) then
            Str(LearnWeightsList[Value - 1] : 4 : 0, HelpString)
          else Str(LearnWeightsList[Value - 1] : 1 : 4, HelpString);
          TextOut(40, 320, IntToStr(Value) + '. Gewicht ' + 'des ' +
            IntToStr(LearnSpinEditWeightChoiceNeuron.Value) + '. Neurons');
          TextOut(40, 335, 'der ' + IntToStr(LearnSpinEditWeightChoiceLayer.Value) +
            '. Schicht = ' + HelpString);
        end;
        { Weight 2 - text}
        with LearnSpinEditWeightChoice2 do begin
          if (LearnWeightsList[Value - 1] > 10) then
            Str(LearnWeightsList[Value - 1] : 4 : 0, HelpString)
          else Str(LearnWeightsList[Value - 1] : 1 : 4, HelpString);
          TextOut(250, 320, IntToStr(Value) + '. Gewicht ' + 'des ' +
            IntToStr(LearnSpinEditWeightChoiceNeuron.Value) + '. Neurons');
          TextOut(250, 335, 'der ' + IntToStr(LearnSpinEditWeightChoiceLayer.Value) +
            '. Schicht = ' + HelpString);
        end;
        { Draw Hinton diagrams sized like chosen weights 1 and 2 of neuron }
        { Hinton diagram of 1st weight }
        RectSide := (Abs(Round(LearnWeightsList[LearnSpinEditWeightChoice1.
          Value - 1] * LearnFactYHinton)));
        OrigoX1 := 115 - Round(RectSide * 1 / 2);
        OrigoY1 := 160 - Round(RectSide * 1 / 2);
        if (LearnWeightsList[LearnSpinEditWeightChoice1.Value - 1] < 0) then begin
          Brush.Color := clBlue;
          FillRect(Rect(OrigoX1, OrigoY1, (OrigoX1 + RectSide), (OrigoY1 + RectSide)));
        end
        else begin
          Brush.Color := clBtnFace;
          FillRect(Rect(OrigoX1, OrigoY1, (OrigoX1 + RectSide), (OrigoY1 + RectSide)));
          Pen.Width := 2;
          Pen.Color := clBlue;
          Rectangle(OrigoX1, OrigoY1, (OrigoX1 + RectSide), (OrigoY1 + RectSide));
        end;
        { Hinton diagram of 2nd weight }
        RectSide := (Abs(Round(LearnWeightsList[LearnSpinEditWeightChoice2.
          Value - 1] * LearnFactYHinton)));
        OrigoX2 := 345 - Round(RectSide * 1 / 2);
        OrigoY2 := 160 - Round(RectSide * 1 / 2);
        if (LearnWeightsList[LearnSpinEditWeightChoice2.Value - 1] < 0) then begin
          Brush.Color := clBlue;
          FillRect(Rect(OrigoX2, OrigoY2, (OrigoX2 + RectSide), (OrigoY2 + RectSide)));
        end
        else begin
          Brush.Color := clBtnFace;
          FillRect(Rect(OrigoX2, OrigoY2, (OrigoX2 + RectSide), (OrigoY2 + RectSide)));
          Pen.Width := 2;
          Pen.Color := clBlue;
          Rectangle(OrigoX2, OrigoY2, (OrigoX2 + RectSide), (OrigoY2 + RectSide));
        end;
      end;
    end;
  end;
end;

procedure TBackPropForm.LearnRadioGroupMethodClick(Sender: TObject);
begin
  { If learn method is kumulative = epochal }
  if (LearnRadioGroupMethod.ItemIndex = 0) then begin
    LearnEditMomentum.Enabled := True;
    LearnEditMomentum.Color := clWindow;
    LearnCheckBoxAdaptiveLearning.Enabled := True;
    LearnCheckBoxAdaptiveLearning.State := cbUnchecked;
    LearnCheckBoxAdaptiveLearningClick(Self);
    { Hand over learn method to net component }
    BackPropBackPropNet.LearnMethod := lmEpochal;
  end
  { If learn method is simple = direct }
  else begin
    LearnEditMomentum.Enabled := False;
    LearnEditMomentum.Color := clBtnFace;
    LearnCheckBoxAdaptiveLearning.Enabled := False;
    LearnCheckBoxAdaptiveLearning.State := cbGrayed;
    LearnCheckBoxAdaptiveLearningClick(Self);
    { Hand over learn method to net component }
    BackPropBackPropNet.LearnMethod := lmDirect;
  end;
end;

procedure TBackPropForm.LearnReadParams;
begin
  { !! Leave order of settings unchanged !! - called methods vary parameters }
  with BackPropBackPropNet do begin
    { Learn method }
    if (LearnMethod = lmEpochal) then LearnRadioGroupMethod.ItemIndex := 0
    else LearnRadioGroupMethod.ItemIndex := 1;
    { CycleLimit }
    LearnEditCycleLimit.Text := IntToStr(CycleLimit);
    { ErrorLimit }
    LearnEditErrorLimit.Text := FloatToStr(ErrorLimit);
    { ParamsMin }
    LearnEditParamsMin.Text := FloatToStr(ParamsMin);
    { ParamsMax }
    LearnEditParamsMax.Text := FloatToStr(ParamsMax);
    { LearnParams }
    if (lpWeights in LearnParams) then LearnCheckBoxLearnWeights.State := cbChecked
    else LearnCheckBoxLearnWeights.State := cbUnchecked;
    if (lpBias in LearnParams) then LearnCheckBoxLearnThresh.State := cbChecked
    else LearnCheckBoxLearnThresh.State := cbUnchecked;
    if (lpSlope in LearnParams) then LearnCheckBoxLearnSlope.State := cbChecked
    else LearnCheckBoxLearnSlope.State := cbUnchecked;
    { Save learned data (weights, bias, slope) in file }
    LearnCheckBoxSave.State := cbUnchecked;
    { LearnRate }
    LearnEditLearnRate.Text := FloatToStr(LearnRate);
    { LearnRate recursive }
    if ((LearnRateDec < 1) or (LearnRateInc > 1)) then
      LearnCheckBoxLearnRateRecursive.State := cbChecked
    else LearnCheckBoxLearnRateRecursive.State := cbUnchecked;
    LearnCheckBoxLearnRateRecursiveClick(Self);
    { Momentum }
    LearnEditMomentum.Text := FloatToStr(Momentum);
    { AdaptiveLearning }
    if ((AdaptConst <> 0) or (AdaptConstFactor <> 1) or (AdaptConstStep <> 0)) then
      LearnCheckBoxAdaptiveLearning.State := cbChecked
    else LearnCheckBoxAdaptiveLearning.State := cbUnchecked;
    LearnCheckBoxAdaptiveLearningClick(Self);
    { Recording of learn data in lists }
    LearnCheckBoxCollectResults.State := cbUnchecked;
    { TopIndex of ListBoxCollectResults }
    LearnComboBoxCollectResults.ItemIndex := 0;
    { Set WeightChoice SpinEdits }
    LearnBitBtnWeightChoiceClick(Self);
    { Write FileName in Edit }
    LearnEditFileLrn.Text := 'Untitled.lrn';
  end;
end;

procedure TBackPropForm.LearnSetDataListGlobalErrorDuringLearning
  (ALearnErrorList: TRealArray);
var
  i: Integer;
  RangeX, RangeY: Double;
begin
  LearnErrorList.Free;
  LearnErrorList := nil;
  LearnErrorList := TRealArray.Create(0);
  if (ALearnErrorList.Count > 0) then begin
    { Transmitt overgiven list to local list }
    for i := 0 to (ALearnErrorList.Count - 1) do LearnErrorList.Add(ALearnErrorList[i]);
    { Get minimum and maximum of curve }
    LearnErrorMinX := 1;
    LearnErrorMaxX := LearnErrorList.Count;
    LearnErrorMinY := LearnErrorList[0];
    LearnErrorMaxY := LearnErrorList[0];
    for i := 0 to (LearnErrorList.Count - 1) do begin
      if (LearnErrorList.Items[i] < LearnErrorMinY) then LearnErrorMinY := LearnErrorList[i];
      if (LearnErrorList.Items[i] > LearnErrorMaxY) then LearnErrorMaxY := LearnErrorList[i];
    end;
    { Calculate ranges for x- and y-axis }
    RangeX := (LearnErrorMaxX - LearnErrorMinX);
    if (RangeX <> 0.0) then LearnFactXDiag := ((LearnPaintBoxDiag.Width - 30) / RangeX)
    else LearnFactXDiag := 0.0;
    RangeY := (LearnErrorMaxY - LearnErrorMinY);
    if (RangeY <> 0.0) then LearnFactYDiag := ((LearnPaintBoxDiag.Height - 60) / RangeY)
    else LearnFactYDiag := 0.0
  end
  else begin
    { Set parameters to zero }
    LearnFactXDiag := 0;
    LearnFactYDiag := 0;
    LearnErrorMaxX := 0;
    LearnErrorMinX := 0;
    LearnErrorMaxY := 0;
    LearnErrorMinY := 0;
  end;
end;

procedure TBackPropForm.LearnSetDataListsWeightsDuringLearning
  (ALearnWeightsList: TRealArray);
var
  i: Integer;
  RangeY: Double;
begin
  LearnWeightsList.Free;
  LearnWeightsList := nil;
  LearnWeightsList := TRealArray.Create(0);
  if (ALearnWeightsList.Count > 0) then begin
    { Transmitt overgiven lists to local lists }
    for i := 0 to (ALearnWeightsList.Count - 1) do LearnWeightsList.Add(ALearnWeightsList[i]);
    { Get minimum and maximum of curve }
    LearnWeightsMinY := LearnWeightsList[0];
    LearnWeightsMaxY := LearnWeightsList[0];
    for i := 0 to (LearnWeightsList.Count - 1) do begin
      if (LearnWeightsList[i] < LearnWeightsMinY) then LearnWeightsMinY := LearnWeightsList[i];
      if (LearnWeightsList[i] > LearnWeightsMaxY) then LearnWeightsMaxY := LearnWeightsList[i];
    end;
    { Calculate range for y-axis }
    RangeY := Abs(LearnWeightsMaxY);
    if ((Abs(LearnWeightsMinY)) > (Abs(LearnWeightsMaxY))) then RangeY := Abs(LearnWeightsMinY);
    if (RangeY <> 0.0) then begin
      LearnFactYBar := (150 / RangeY);
      LearnFactYHinton := (110 / RangeY);
    end
    else begin
      LearnFactYBar := 0.0;
      LearnFactYHinton := 0.0;
    end;
  end
  else begin
    { Set parameters to zero }
    LearnFactYBar := 0;
    LearnFactYHinton := 0;
    LearnWeightsMinY := 0;
    LearnWeightsMaxY := 0;
  end;
end;

procedure TBackPropForm.LearnShowBoxes;
begin
  { Visualize boxes, but not diagrams }
  LearnRadioGroupMethod.Visible := True;
  LearnGroupBoxLearnProcess.Visible := True;
  LearnGroupBoxLearnParams.Visible := True;
  LearnGroupBoxLimit.Visible := True;
  LearnGroupBoxLearnRateRecursive.Visible := True;
  LearnGroupBoxAdaptiveLearning.Visible := True;
  LearnGroupBoxLearnParamsInit.Visible := True;
  LearnGroupBoxCollectResults.Visible := True;
  { Elements belonging to diagrams }
  LearnGroupBoxWeightChoice.Visible := False;
  LearnPaintBoxBar.Visible := False;
  LearnPaintBoxDiag.Visible := False;
  LearnPaintBoxHinton.Visible := False;
  LearnPanelGlobalError.Visible := False;
  LearnPanelWeights.Visible := False;
end;

procedure TBackPropForm.LearnSpeedButtonBarClick(Sender: TObject);
begin
  { Make visible Bar diagram }
  if (LearnSpeedButtonBar.Down = True) then begin
    LearnHideBoxes;
    LearnGroupBoxWeightChoice.Visible := True;
    LearnPaintBoxBar.Visible := True;
    LearnPanelWeights.Visible := True;
  end
  else LearnShowBoxes;
end;

procedure TBackPropForm.LearnSpeedButtonDiagClick(Sender: TObject);
begin
  { Make visible Error diagram }
  if (LearnSpeedButtonDiag.Down = True) then begin
    LearnHideBoxes;
    LearnPaintBoxDiag.Visible := True;
    LearnPanelGlobalError.Visible := True;
  end
  else LearnShowBoxes;
end;

procedure TBackPropForm.LearnSpeedButtonHintonClick(Sender: TObject);
begin
  { Make visible Hinton diagram }
  if (LearnSpeedButtonHinton.Down = True) then begin
    LearnHideBoxes;
    LearnGroupBoxWeightChoice.Visible := True;
    LearnPaintBoxHinton.Visible := True;
    LearnPanelWeights.Visible := True;
  end
  else LearnShowBoxes;
end;

procedure TBackPropForm.LearnSpinEditWeightChoice1Change(Sender: TObject);
var
  HelpValue: Integer;
begin
  with LearnSpinEditWeightChoice1 do try
    HelpValue := Value;
    { Check the given value for conflict with net component }
    if ((HelpValue < 1) or (HelpValue > BackPropBackPropNet.Items[LearnSpinEditWeightChoiceLayer.Value - 2].
      Items[LearnSpinEditWeightChoiceNeuron.Value - 1].Weights.Count)) then begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht zeichnen. Wert fr erstes Gewicht prfen! ' + #13#10 +
        '(mindestens 1, hchstens Gewichtanzahl des Neurons)', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Gewicht muss Integer sein!', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnSpinEditWeightChoice2Change(Sender: TObject);
var
  HelpValue: Integer;
begin
  with LearnSpinEditWeightChoice2 do try
    HelpValue := Value;
    if ((HelpValue < 1) or (HelpValue > BackPropBackPropNet.Items[LearnSpinEditWeightChoiceLayer.Value - 2].
      Items[LearnSpinEditWeightChoiceNeuron.Value - 1].Weights.Count)) then begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht zeichnen. Wert fr zweites Gewicht prfen! ' + #13#10 +
        '(mindestens 1, hchstens Gewichtanzahl des Neurons)', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Gewicht muss Integer sein!', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnSpinEditWeightChoiceLayerChange(Sender: TObject);
var
  HelpValue: Integer;
begin
  with LearnSpinEditWeightChoiceLayer do try
    HelpValue := Value;
    { Check the given value for conflict with net component }
    if ((HelpValue < 2) or (HelpValue > (BackPropBackPropNet.LayerNo + 1))) then begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht zeichnen. Wert fr Schicht prfen! ' + #13#10 +
        '(mindestens 2, hchstens Schichtanzahl im Netz)', mtError, [mbOk], 0);
      Value := 2;
      SetFocus;
    end
    else begin
      with LearnSpinEditWeightChoiceNeuron do begin
        { Change number of neurons depending on chosen layer }
        MaxValue := BackPropBackPropNet.Items[LearnSpinEditWeightChoiceLayer.Value - 2].NeuronNo;
        if (MaxValue > 1) then MinValue := 1 else MinValue := 0;
        Value := 1;
        LearnSpinEditWeightChoiceNeuronChange(Self);
      end;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Schicht muss Integer sein!', mtError, [mbOk], 0);
      Value := 2;
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.LearnSpinEditWeightChoiceNeuronChange(Sender: TObject);
var
  HelpValue: Integer;
begin
  with LearnSpinEditWeightChoiceNeuron do try
    HelpValue := Value;
    { Check the given value for conflict with net component }
    if ((HelpValue < 1) or (HelpValue > BackPropBackPropNet.
      Items[LearnSpinEditWeightChoiceLayer.Value - 2].NeuronNo)) then begin
      MessageBeep(0);
      MessageDlg('Kann Diagramm nicht zeichnen. Wert fr Neuron prfen! ' + #13#10 +
        '(mindestens 1, hchstens Neuronenanzahl in Schicht)', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end
    else begin
      { Change number of weights depending on chosen neuron }
      { Weight 1 }
      with LearnSpinEditWeightChoice1 do begin
        MaxValue := BackPropBackPropNet.Items[LearnSpinEditWeightChoiceLayer.Value - 2].
          Items[LearnSpinEditWeightChoiceNeuron.Value - 1].Weights.Count;
        if (MaxValue > 1) then MinValue := 1 else MinValue := 0;
        Value := 1;
      end;
      { Weight 2 }
      with LearnSpinEditWeightChoice2 do begin
        MaxValue := BackPropBackPropNet.Items[LearnSpinEditWeightChoiceLayer.
          Value - 2].Items[LearnSpinEditWeightChoiceNeuron.Value - 1].Weights.Count;
        if (MaxValue > 1) then MinValue := 1 else MinValue := 0;
        Value := 1;
      end;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Neuron muss grsser 1, aber hchstens gleich ' +
        'der Anzahl der Neuronen in Schicht sein!', mtError, [mbOk], 0);
      Value := 1;
      SetFocus;
    end;
  end;
end;

{ TabSheet 'Pattern' following }

procedure TBackPropForm.PattBitBtnDelPattClick(Sender: TObject);
begin
  with PattComboBoxPattCurr do begin
    { Delete last pattern of pattern set? }
    if (Items.Count = 1) then begin
      MessageBeep(0);
      if (MessageDlg('Nur noch 1 Muster im Set. Lschen und Set neu setzen?',
        mtConfirmation, [mbYes, mbNo], 0) = mrYes) then begin
        { Delete all existing patterns }
        with BackPropBackPropNet do begin
          PatternSet.DeleteAll;
          PatternSet.PatternNo := 1;
        end;
        { Delete number of patterns in ComboBox }
        Clear;
        { Write new pattern number into ComboBox }
        Items.Add(IntToStr(Items.Count + 1));
        ItemIndex := (Items.Count - 1);
        { Set dimension in PattPanelPatternSetDimensionInp/Targ }
        with BackPropBackPropNet do begin
          PattEditPatternSetDimInp.Text := IntToStr(PatternSet.InputNo);
          PattEditPatternSetDimTarg.Text := IntToStr(PatternSet.OutputNo);
        end;
      end;
    end
    else if (Items.Count >= 2) then begin
      { Delete pattern in net component }
      BackPropBackPropNet.PatternSet.Delete(ItemIndex);
      { Change data in ComboBox }
      Items.Delete(Items.Count - 1);
      ItemIndex := (Items.Count - 1);
    end;
    { Setting panel showing total number of patterns }
    PattPanelNrPatt.Caption := IntToStr(Items.Count);
  end;
  { Set values in other boxes and elements }
  PattComboBoxPattCurrChange(Self);
end;

procedure TBackPropForm.PattBitBtnNewInitClick(Sender: TObject);
begin
  with BackPropBackPropNet.PatternSet do begin
    { Delete all existing patterns }
    DeleteAll;
    { Set new PatternSet }
    PatternNo := 1;
    InputNo := BackPropBackPropNet.InputNo;
    OutputNo := BackPropBackPropNet.Items[(BackPropBackPropNet.LayerNo - 1)].NeuronNo;
    { Delete number of patterns in ComboBox }
    PattComboBoxPattCurr.Clear;
    { Write new pattern number into ComboBox }
    PattComboBoxPattCurr.Items.Add(IntToStr(PattComboBoxPattCurr.Items.Count + 1));
    PattComboBoxPattCurr.ItemIndex := (PattComboBoxPattCurr.Items.Count - 1);
    { Set dimension in PattPanelPatternSetDimensionInp/Targ }
    PattEditPatternSetDimInp.Text := IntToStr(InputNo);
    PattEditPatternSetDimTarg.Text := IntToStr(OutputNo);
  end;
  { Setting panel showing total number of patterns }
  PattPanelNrPatt.Caption := IntToStr(PattComboBoxPattCurr.Items.Count);
  PattComboBoxPattCurrChange(Self);
  { Write file name in Edit }
  PattEditFilePtn.Text := 'Untitled.ptn';
end;

procedure TBackPropForm.PattBitBtnNewPattClick(Sender: TObject);
begin
  with PattComboBoxPattCurr do begin
    { Set ComboBox Items/Index }
    Items.Add(IntToStr(Items.Count + 1));
    ItemIndex := (Items.Count - 1);
    {Create new pattern at the end of the set }
    with BackPropBackPropNet.PatternSet do PatternNo := (PatternNo + 1);
    { Setting panel showing total number of patterns }
    PattPanelNrPatt.Caption := IntToStr(Items.Count);
  end;
  PattComboBoxPattCurrChange(Self);
end;

procedure TBackPropForm.PattBitBtnOpenPatternSetClick(Sender: TObject);
var
  i: Integer;
begin
  with BackPropNetFileDialog do try
    DialogMode := dmOpen;
    Title := 'ffnen einer MusterSetDatei';
    { Filter on Pattern }
    FilterIndex := 5;
    FileName := PattEditFilePtn.Text;
    DefaultExt := 'ptn';
    if (Execute = True) then begin
      BackPropBackPropNet.LoadFromFile(FileName);
      { Write FileName in PattEditFilePtn }
      PattEditFilePtn.Text := ExtractFileName(FileName);
    end;
    with PattComboBoxPattCurr do begin
      { Delete number of patterns in ComboBox }
      Clear;
      { Write new pattern number into ComboBox }
      for i := 0 to (BackPropBackPropNet.PatternSet.PatternNo - 1) do
        Items.Add(IntToStr(Items.Count + 1));
      ItemIndex := (Items.Count - 1);
      { Set dimension in PattPanelPatternSetDimensionInp/Targ }
      with BackPropBackPropNet do begin
        PattEditPatternSetDimInp.Text := IntToStr(PatternSet.InputNo);
        PattEditPatternSetDimTarg.Text := IntToStr(PatternSet.OutputNo);
      end;
      { Setting panel showing total number of patterns }
      PattPanelNrPatt.Caption := IntToStr(Items.Count);
    end;
    PattComboBoxPattCurrChange(Self);
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message, mtError, [mbOK], 0);
    end;
  end;
end;

procedure TBackPropForm.PattBitBtnPatternSetDimInpDecClick(Sender: TObject);
begin
  with BackPropBackPropNet.PatternSet do
    if (InputNo > 1) then begin
      InputNo := (InputNo - 1);
      PattEditPatternSetDimInp.Text := IntToStr(InputNo);
      PattComboBoxPattCurrChange(Self);
    end
    else begin
      MessageBeep(0);
      MessageDlg('Dimension des Eingangsvektors kann nicht < 1 sein!', mtWarning, [mbOk], 0);
    end;
end;

procedure TBackPropForm.PattBitBtnPatternSetDimInpIncClick(Sender: TObject);
begin
  with BackPropBackPropNet.PatternSet do begin
    InputNo := (InputNo + 1);
    PattEditPatternSetDimInp.Text := IntToStr(InputNo);
  end;
  PattComboBoxPattCurrChange(Self);
end;

procedure TBackPropForm.PattBitBtnPatternSetDimTargDecClick(Sender: TObject);
begin
  with BackPropBackPropNet.PatternSet do
    if (OutputNo > 1) then begin
      OutputNo := (OutputNo - 1);
      PattEditPatternSetDimTarg.Text := IntToStr(OutputNo);
      PattComboBoxPattCurrChange(Self);
    end
    else begin
      MessageBeep(0);
      MessageDlg('Dimension des Zielvektors kann nicht < 1 sein!', mtWarning, [mbOk], 0);
    end;
end;

procedure TBackPropForm.PattBitBtnPatternSetDimTargIncClick(Sender: TObject);
begin
  with BackPropBackPropNet.PatternSet do begin
    OutputNo := (OutputNo + 1);
    PattEditPatternSetDimTarg.Text := IntToStr(OutputNo);
  end;
  PattComboBoxPattCurrChange(Self);
end;

procedure TBackPropForm.PattBitBtnSavePatternSetClick(Sender: TObject);
begin
  with BackPropNetFileDialog do try
    DialogMode := dmSave;
    Title := 'Speichern einer MusterSetDatei';
    { Filter on Pattern }
    FilterIndex := 5;
    FileName := PattEditFilePtn.Text;
    DefaultExt := 'ptn';
    if Execute then begin
      BackPropBackPropNet.SaveToFile(FileName);
      PattEditFilePtn.Text := ExtractFileName(FileName);
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TBackPropForm.PattComboBoxPattCurrChange(Sender: TObject);
var
  i: Integer;
begin
  { Set listboxes }
  PattListBoxInp.Clear;
  PattListBoxTarg.Clear;
  with PattComboBoxPattCurr do
    if (ItemIndex >= 0) then begin
      { Setting labels showing size of input/target vector of pattern }
      PattLabelListInp.Caption := 'EingangsVektor ( ' +
        IntToStr(BackPropBackPropNet.PatternSet.Items[ItemIndex].Inputs.Count) + ' )';
      PattLabelListTarg.Caption := 'ZielVektor ( ' +
        IntToStr(BackPropBackPropNet.PatternSet.Items[ItemIndex].Outputs.Count) + ' )';
      { Fill ListBoxes if input/target vectors exist }
      if (BackPropBackPropNet.PatternSet.Items[ItemIndex].Inputs.Count >= 1) then
        for i := 0 to (BackPropBackPropNet.PatternSet.Items[ItemIndex].Inputs.Count - 1) do
          PattListBoxInp.Items.Add(FloatToStr(BackPropBackPropNet.PatternSet.Items[ItemIndex].Inputs[i]));
      if (BackPropBackPropNet.PatternSet.Items[ItemIndex].Outputs.Count >= 1) then
        for i := 0 to (BackPropBackPropNet.PatternSet.Items[ItemIndex].Outputs.Count - 1) do
          PattListBoxTarg.Items.Add(FloatToStr(BackPropBackPropNet.PatternSet.Items[ItemIndex].Outputs[i]));
      PattListBoxInp.ItemIndex := 0;
      PattListBoxTarg.ItemIndex := 0;
    end
    else begin
      { Setting labels showing emptiness of input/target vectors }
      PattLabelListInp.Caption := 'EingangsVektor';
      PattLabelListTarg.Caption := 'ZielVektor';
      { Set ListBoxes to empty }
      PattListBoxInp.ItemIndex := -1;
      PattListBoxTarg.ItemIndex := -1;
    end;
end;

procedure TBackPropForm.PattEditPatternSetDimInpExit(Sender: TObject);
var
  HelpValue: Integer;
begin
  with PattEditPatternSetDimInp do try
    HelpValue := StrToInt(Text);
    if (HelpValue < 1) then begin
      MessageBeep(0);
      MessageDlg('Dimension des Eingangs kann nicht < 1 sein!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.PatternSet.InputNo);
      SetFocus;
    end
    else begin
      BackPropBackPropNet.PatternSet.InputNo := HelpValue;
      PattComboBoxPattCurrChange(Self);
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Dimension des Eingangsvektors prfen!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.PatternSet.InputNo);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.PattEditPatternSetDimInpKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattEditPatternSetDimInpExit(Self);
end;

procedure TBackPropForm.PattEditPatternSetDimTargExit(Sender: TObject);
var
  HelpValue: Integer;
begin
  with PattEditPatternSetDimTarg do try
    HelpValue := StrToInt(Text);
    if (HelpValue < 1) then begin
      MessageBeep(0);
      MessageDlg('Dimension des Zielvektors kann nicht < 1 sein!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.PatternSet.OutputNo);
      SetFocus;
    end
    else begin
      BackPropBackPropNet.PatternSet.OutputNo := HelpValue;
      PattComboBoxPattCurrChange(Self);
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Dimension des Zielvektors prfen!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.PatternSet.OutputNo);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.PattEditPatternSetDimTargKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattEditPatternSetDimTargExit(Self);
end;

procedure TBackPropForm.PattEditValInpCurrExit(Sender: TObject);
begin
  with PattEditValInpCurr do try
    BackPropBackPropNet.PatternSet.Items[PattComboBoxPattCurr.ItemIndex].
      Inputs[PattListBoxInp.ItemIndex] := StrToFloat(Text);
    PattComboBoxPattCurrChange(Self);
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Eingangsvektor prfen!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.PatternSet.
        Items[PattComboBoxPattCurr.ItemIndex].Inputs[PattListBoxInp.ItemIndex]);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.PattEditValInpCurrKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattEditValInpCurrExit(Self);
end;

procedure TBackPropForm.PattEditValTargCurrExit(Sender: TObject);
begin
  with PattEditValTargCurr do try
    BackPropBackPropNet.PatternSet.Items[PattComboBoxPattCurr.ItemIndex].
      Outputs[PattListBoxTarg.ItemIndex] := StrToFloat(Text);
    PattComboBoxPattCurrChange(Self);
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Zielvektor prfen!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.PatternSet.
        Items[PattComboBoxPattCurr.ItemIndex].Outputs[PattListBoxTarg.ItemIndex]);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.PattEditValTargCurrKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattEditValTargCurrExit(Self);
end;

procedure TBackPropForm.PattListBoxInpExit(Sender: TObject);
begin
  with PattListBoxInp do
    PattEditValInpCurr.Text := Items[ItemIndex];
  PattEditValInpCurr.SetFocus;
end;

procedure TBackPropForm.PattListBoxInpKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattListBoxInpExit(Self);
end;

procedure TBackPropForm.PattListBoxTargExit(Sender: TObject);
begin
  with PattListBoxTarg do
    PattEditValTargCurr.Text := Items[ItemIndex];
  PattEditValTargCurr.SetFocus;
end;

procedure TBackPropForm.PattListBoxTargKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattListBoxTargExit(Self);
end;

{ TabSheet 'Pruning' following }

procedure TBackPropForm.PrunBitBtnBreakClick(Sender: TObject);
begin
  { Interrupt pruning process }
  BackPropBackPropNet.Break := True;
end;

procedure TBackPropForm.PrunBitBtnMinConfigClick(Sender: TObject);
var
  k: Integer;
begin
  with BackPropBackPropNet do begin
    { Reset structure and all parameters of component BackPropBackPropNet }
    ResetAll;
    { Transmit net with minimal structure to component BackPropBackPropNet }
    LayerNo := (TRealArray(PrunNetListNetConfig.Items[PrunNetMinStructure]).Count + 1);
    InputNo := PrunStoreInputNo;
    if (LayerNo > 1) then
      for k := 0 to (LayerNo - 2) do
        Items[k].NeuronNo := Round(TRealArray(PrunNetListNetConfig.
          Items[PrunNetMinStructure]).Items[k]);
    Items[LayerNo - 1].NeuronNo := PrunStoreOutputNo;
    { Free list with configuration of optimized nets }
    PrunNetListNetConfig.Free;
    PrunNetListNetConfig := nil;
    { Write LayerNo and NeuronNo in page 'Config' }
    if (ConfigSpinEditNrLay.Value = (LayerNo + 1)) then begin
      ConfigSpinEditNrLay.Value := (LayerNo + 1);
      ConfigSpinEditNrLayChange(Self);
    end
    else ConfigSpinEditNrLay.Value := (LayerNo + 1);
  end;
  { Disable elements }
  PrunGroupBoxConfig.Enabled := False;
  { Confirmation that chosen structure has been created }
  MessageBeep(0);
  MessageDlg('Struktur des gewhlten Netzes hergestellt. ' + #13#10 +
    'Ein-/Ausgangsneuronen nicht verndert, um Muster beibehalten zu knnen.',
    mtInformation, [mbOk], 0);
end;

procedure TBackPropForm.PrunBitBtnMinCyclesClick(Sender: TObject);
var
  k: Integer;
begin
  with BackPropBackPropNet do begin
    { Reset structure and all parameters of component BackPropBackPropNet }
    ResetAll;
    { Transmit net with minimal learn cycles to component BackPropBackPropNet }
    LayerNo := (TRealArray(PrunNetListNetConfig.Items[PrunNetMinCycles]).Count + 1);
    InputNo := PrunStoreInputNo;
    if (LayerNo > 1) then
      for k := 0 to (LayerNo - 2) do
        Items[k].NeuronNo := Round(TRealArray(PrunNetListNetConfig.
          Items[PrunNetMinCycles]).Items[k]);
    Items[LayerNo - 1].NeuronNo := PrunStoreOutputNo;
    { Free list with configuration of optimized nets }
    PrunNetListNetConfig.Free;
    PrunNetListNetConfig := nil;
    { Write LayerNo and NeuronNo in page 'Config' }
    if (ConfigSpinEditNrLay.Value = (LayerNo + 1)) then begin
      ConfigSpinEditNrLay.Value := (LayerNo + 1);
      ConfigSpinEditNrLayChange(Self);
    end
    else ConfigSpinEditNrLay.Value := (LayerNo + 1);
  end;
  { Disable elements }
  PrunGroupBoxConfig.Enabled := False;
  { Confirmation that chosen structure has been created }
  MessageBeep(0);
  MessageDlg('Struktur des gewhlten Netzes hergestellt. ' + #13#10 +
    'Ein-/Ausgangsneuronen nicht verndert, um Muster beibehalten zu knnen.',
    mtInformation, [mbOk], 0);
end;

procedure TBackPropForm.PrunBitBtnPrimaryClick(Sender: TObject);
var
  k: Integer;
begin
  with BackPropBackPropNet do begin
    { Reset structure and all parameters of component BackPropBackPropNet }
    ResetAll;
    { Transmit primary structure to component BackPropBackPropNet }
    LayerNo := (TRealArray(PrunNetListNetConfig.Items[0]).Count + 1);
    InputNo := PrunStoreInputNo;
    if (LayerNo > 1) then
      for k := 0 to (LayerNo - 2) do
        Items[k].NeuronNo := Round(TRealArray(PrunNetListNetConfig.Items[0]).Items[k]);
    Items[LayerNo - 1].NeuronNo := PrunStoreOutputNo;
    { Free list with configuration of optimized nets }
    PrunNetListNetConfig.Free;
    PrunNetListNetConfig := nil;
    { Write LayerNo and NeuronNo in page 'Config' }
    if (ConfigSpinEditNrLay.Value = (LayerNo + 1)) then begin
      ConfigSpinEditNrLay.Value := (LayerNo + 1);
      ConfigSpinEditNrLayChange(Self);
    end
    else ConfigSpinEditNrLay.Value := (LayerNo + 1);
  end;
  { Disable elements }
  PrunGroupBoxConfig.Enabled := False;
  { Confirmation that chosen structure has been created }
  MessageBeep(0);
  MessageDlg('Struktur des gewhlten Netzes hergestellt. ' + #13#10 +
    'Ein-/Ausgangsneuronen nicht verndert, um Muster beibehalten zu knnen.',
    mtInformation, [mbOk], 0);
end;

procedure TBackPropForm.PrunBitBtnStartClick(Sender: TObject);

  { Help procedure to free all local lists of previous pruning sessions }
  procedure PrunFreeAllLocalPrunLists;
  var
    n: Integer;
  begin
    if Assigned(PrunLayerListNeuronSumWeights) then begin
      with PrunLayerListNeuronSumWeights do begin
        if (Count > 0) then begin
          for n := 0 to (Count - 1) do begin
            TRealArray(Items[n]).Free;
          end;
        end;
        Free;
        PrunLayerListNeuronSumWeights := nil;
      end;
    end;
    if Assigned(PrunLayerListLayerSumWeights) then begin
      with PrunLayerListLayerSumWeights do Free;
      PrunLayerListLayerSumWeights := nil;
    end;
    if Assigned(PrunLayerListNeuronPower) then begin
      with PrunLayerListNeuronPower do begin
        if (Count > 0) then begin
          for n := 0 to (Count - 1) do begin
            TRealArray(Items[n]).Free;
          end;
        end;
        Free;
        PrunLayerListNeuronPower := nil;
      end;
    end;
    if Assigned(PrunLayerListLayerSumPower) then begin
      with PrunLayerListLayerSumPower do Free;
      PrunLayerListLayerSumPower := nil;
    end;
    if Assigned(PrunLayerListNeuronProdRelWeightsPower) then begin
      with PrunLayerListNeuronProdRelWeightsPower do begin
        if (Count > 0) then begin
          for n := 0 to (Count - 1) do begin
            TRealArray(Items[n]).Free;
          end;
        end;
        Free;
        PrunLayerListNeuronProdRelWeightsPower := nil;
      end;
    end;
    if Assigned(PrunLayerListLayerMeanValProd) then begin
      with PrunLayerListLayerMeanValProd do Free;
      PrunLayerListLayerMeanValProd := nil;
    end;
  end;

var
  i: Integer; { Index for neuron }
  j: Integer; { Index for weight = input of neuron }
  k: Integer; { Index for layer }
  { Weights and Power of neuron and layer }
  NeuronSumWeights: Double;
  LayerSumWeights: Double;
  LayerSumPower: Double;
  { Relative values (normalized) and product of weight and power }
  RelNeuronWeights: Double;
  RelNeuronPower: Double;
  SumAllProd: Double;
  { Percentage of influence of neuron within net}
  PercInflNeuron: Double;
  { Compare structures of current net with net optimized before }
  CompareStruct: Boolean;
  { Minimum of all LearnCycles }
  MinLearnCycles: Integer;
  { Number of optimized nets = optimization sessions }
  OptNetNo: Integer;
  { LayerLists containing values representative for one layer }
  PrunLayerListNetConfig: TRealArray;
begin
  MessageBeep(0);
  if (MessageDlg('Optimieren kann - je nach Lernschrittanzahl und Konfiguration - lnger dauern. ' +
    'Vorgang fortsetzen?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
    with BackPropBackPropNet do try
      { Set WaitCursor }
      Screen.Cursor := crBackPropWaitCursor;
      { Set status }
      BackPropSetStatus(1, 'ParameterCheck wird durchgefhrt');
      BackPropStatusBar.Refresh;
      { Check LayerNo }
      if (LayerNo < 2) then begin
        MessageBeep(0);
        raise Exception.Create('Optimierung nur durchfhrbar, wenn Anzahl ' +
          'der Schichten mindestens 3 betrgt (incl. Verteilungsschicht)!');
      end;
      { Set procedures for events within net component }
      OnExit := nil;
      OnInterval := nil;
      { Store number of layers, inputs and outputs of original net }
      PrunStoreLayerNo := LayerNo;
      PrunStoreInputNo := InputNo;
      PrunStoreOutputNo := Items[(LayerNo - 1)].NeuronNo;
      { Set status }
      BackPropSetStatus(2, 'Netz wird belehrt');
      BackPropStatusBar.Refresh;
      { Reset weights and train net }
      ResetAll;
      Learn;
      { Set status }
      BackPropSetStatus(1, 'Alte Listen werden gelscht');
      BackPropStatusBar.Refresh;
      { Free existing lists of previous pruning }
      PrunFreeAllPrunLists;
      { Create net lists for pruning }
      PrunNetListNetConfig := TList.Create;
      PrunNetListLearnCycles := TRealArray.Create(0);
      { Initialization }
      OptNetNo := 0;
      { Set status }
      BackPropSetStatus(2, 'Optimierung luft');
      BackPropStatusBar.Refresh;
      { Repeat until no more changes in net structure }
      repeat
        { Create layer list for global PrunNetListNetConfig }
        PrunLayerListNetConfig := TRealArray.Create(0);
        { Only for all hidden layers do write neuron number in list }
        for k := 0 to (LayerNo - 2) do PrunLayerListNetConfig.Add(Items[k].NeuronNo);
        { Store configuration and learn cycles of current net }
        PrunNetListNetConfig.Add(PrunLayerListNetConfig);
        PrunNetListLearnCycles.Add(Cycles);
        { Free local lists }
        PrunFreeAllLocalPrunLists;
        { Create local lists }
        PrunLayerListNeuronSumWeights := TList.Create;
        PrunLayerListLayerSumWeights := TRealArray.Create(0);
        PrunLayerListNeuronPower := TList.Create;
        PrunLayerListLayerSumPower := TRealArray.Create(0);
        PrunLayerListNeuronProdRelWeightsPower := TList.Create;
        PrunLayerListLayerMeanValProd := TRealArray.Create(0);
        { Calculating weights and power of neurons/layers }
        for k := 0 to (LayerNo - 2) do begin
          { Create NeuronLists }
          PrunNeuronListNeuronSumWeights := TRealArray.Create(0);
          PrunNeuronListNeuronPower := TRealArray.Create(0);
          LayerSumWeights := 0.0;
          LayerSumPower := 0.0;
          { For all neurons in layer do }
          for i := 0 to (Items[k].NeuronNo - 1) do begin
            { Sum up all input weights of neuron }
            NeuronSumWeights := 0;
            { For all weights of neuron do }
            for j := 0 to (Items[k].Items[i].Weights.Count  - 1) do
              NeuronSumWeights := (NeuronSumWeights + Items[k][i].Weights[j]);
            PrunNeuronListNeuronSumWeights.Add(NeuronSumWeights);
            { Sum up all neurons' weights for the whole layer }
            LayerSumWeights := (LayerSumWeights + NeuronSumWeights);
            { Store value for power of single neurons }
            PrunNeuronListNeuronPower.Add(Items[k][i].PowerAll);
            { Sum up all neurons' power for the whole layer }
            LayerSumPower := (LayerSumPower + Items[k][i].PowerAll);
          end;
          PrunLayerListNeuronSumWeights.Add(PrunNeuronListNeuronSumWeights);
          PrunLayerListLayerSumWeights.Add(LayerSumWeights);
          PrunLayerListNeuronPower.Add(PrunNeuronListNeuronPower);
          PrunLayerListLayerSumPower.Add(LayerSumPower);
        end;
        { Calculating further values }
        for k := 0 to (LayerNo - 2) do begin
          { Create NeuronLists }
          PrunNeuronListNeuronProdRelWeightsPower := TRealArray.Create(0);
          SumAllProd := 0.0;
          { For all neurons in layer do }
          for i := 0 to (Items[k].NeuronNo - 1) do begin
            RelNeuronWeights := (TRealArray(PrunLayerListNeuronSumWeights.
              Items[k]).Items[i] / PrunLayerListLayerSumWeights[k]);
            RelNeuronPower := (TRealArray(PrunLayerListNeuronPower.Items[k]).
              Items[i] / PrunLayerListLayerSumPower[k]);
            PrunNeuronListNeuronProdRelWeightsPower.Add(RelNeuronWeights * RelNeuronPower);
            SumAllProd := (SumAllProd + (RelNeuronWeights * RelNeuronPower));
          end;
          PrunLayerListNeuronProdRelWeightsPower.Add(PrunNeuronListNeuronProdRelWeightsPower);
          { Mean value all products divided by NeuronNo in layer }
          PrunLayerListLayerMeanValProd.Add(SumAllProd /
            TRealArray(PrunNetListNetConfig.Items[OptNetNo]).Items[k]);
        end;
        for k := (LayerNo - 2) downto 0 do begin
          { For all neurons in layer do }
          for i := (Items[k].NeuronNo - 1) downto 0 do begin
            { Get percentage for influence of each neuron }
            PercInflNeuron := Round(TRealArray(PrunLayerListNeuronProdRelWeightsPower.
              Items[k]).Items[i] * 100 / PrunLayerListLayerMeanValProd[k]);
            { Count neurons with influence smaller than given percentage }
            if ((PercInflNeuron < PrunSpinEditLimit.Value) and
              (Items[k].NeuronNo > 1)) then Items[k].DeleteNeuron(i);
          end;
          { Delete hidden layer when just 1 neuron or none is left }
          if (Items[k].NeuronNo < 2) then DeleteLayer(k);
        end;
        { Reset weights and train net }
        ResetAll;
        Learn;
        { Is there still any modification in structure? }
        CompareStruct := True;
        { Is LayerNo just 1? }
        if (LayerNo < 2) then begin
          { Create layer list for global PrunNetListNetConfig }
          PrunLayerListNetConfig := TRealArray.Create(0);
          { Store empty hidden configuration and learn cycles of current net }
          PrunNetListNetConfig.Add(PrunLayerListNetConfig);
          PrunNetListLearnCycles.Add(BackPropBackPropNet.Cycles);
          { Set variable to finish pruning as only 1 layer is left in net }
          CompareStruct := True;
          { Increment number of optimization cycles }
          Inc(OptNetNo);
        end
        else begin
          { Does current net have less layers than the net modified before? }
          if (TRealArray(PrunNetListNetConfig.Items[OptNetNo]).Count > (LayerNo - 1)) then begin
            CompareStruct := False;
            { Increment number of optimization cycles }
            Inc(OptNetNo);
          end
          else begin
            { Does current net have less neurons than the net modified before? }
            for k := 0 to (LayerNo - 2) do
              if (CompareStruct = True) and (Round(TRealArray(PrunNetListNetConfig.
                Items[OptNetNo]).Items[k]) > Items[k].NeuronNo) then begin
                CompareStruct := False;
                { Increment number of optimization cycles }
                Inc(OptNetNo);
              end;
          end;
        end;
      until (CompareStruct = True);
      { Free all local lists used in pruning algorithm }
      PrunFreeAllLocalPrunLists;
      { Get optimal net configuration with structure as criterion }
      PrunNetMinStructure := OptNetNo;
      { Get optimal net configuration with learning cycles as criterion }
      MinLearnCycles := Round(PrunNetListLearnCycles[0]);
      PrunNetMinCycles := 0;
      for i := 0 to (OptNetNo) do
        if (PrunNetListLearnCycles[i] < MinLearnCycles) then begin
          MinLearnCycles := Round(PrunNetListLearnCycles[i]);
          PrunNetMinCycles := i;
        end;
      { Write name and number of optimized net configurations }
      with PrunStringGridNeuronNo do begin
        RowCount := (PrunStoreLayerNo + 4);
        for i := 0 to (ColCount - 1) do
          for j := 0 to (RowCount - 1) do Cells[i, j] := '';
        { Write values into StringGrid }
        Cells[0, 0] := 'Optimierte Netze [' + IntToStr(OptNetNo) + ']';
        Cells[1, 0] := 'Konfig Original [0]';
        Cells[2, 0] := 'Konfig MinStruktur [' + IntToStr(PrunNetMinStructure) + ']';
        Cells[3, 0] := 'Konfig MinLernschritte [' + IntToStr(PrunNetMinCycles) + ']';
        { Print learning cycles for each configuration }
        Cells[0, 1] := 'Lernschritte';
        Cells[1, 1] := FloatToStr(PrunNetListLearnCycles[0]);
        Cells[2, 1] := FloatToStr(PrunNetListLearnCycles[PrunNetMinStructure]);
        Cells[3, 1] := FloatToStr(PrunNetListLearnCycles[PrunNetMinCycles]);
        { Print LayerNo for each configuration }
        Cells[0, 2] := 'Schichten';
        Cells[1, 2] := FloatToStr(TRealArray(PrunNetListNetConfig.Items[0]).Count + 2);
        Cells[2, 2] := FloatToStr(TRealArray(PrunNetListNetConfig.Items[PrunNetMinStructure]).Count + 2);
        Cells[3, 2] := FloatToStr(TRealArray(PrunNetListNetConfig.Items[PrunNetMinCycles]).Count + 2);
        { Print neuron number of input layer = net inputs }
        for i := 0 to PrunStoreLayerNo do Cells[0, (i + 3)] := 'Neuronen';
        { Config Orig }
        { Print neuron number of input layer }
        Cells[1, 3] := IntToStr(PrunStoreInputNo);
        { Print neuron number of hidden layers }
        { Print neuron number of primary configuration }
        for k := 0 to (TRealArray(PrunNetListNetConfig.Items[0]).Count - 1) do
          Cells[1, (k + 4)] := FloatToStr(TRealArray(PrunNetListNetConfig.Items[0]).Items[k]);
        { Print neuron number of output layer }
        Cells[1, (TRealArray(PrunNetListNetConfig.Items[0]).Count + 4)] := IntToStr(PrunStoreOutputNo);
        { Config MinStruct }
        { Print neuron number of input layer }
        Cells[2, 3] := IntToStr(PrunStoreInputNo);
        { Print neuron number of hidden layers }
        if (TRealArray(PrunNetListNetConfig.Items[PrunNetMinStructure]).Count > 0) then
          for k := 0 to (TRealArray(PrunNetListNetConfig.Items[PrunNetMinStructure]).Count - 1) do
            { Print neuron number of primary configuration }
            Cells[2, (k + 4)] := FloatToStr(TRealArray(PrunNetListNetConfig.Items[PrunNetMinStructure]).Items[k]);
        { Print neuron number of output layer }
        Cells[2, (TRealArray(PrunNetListNetConfig.Items[PrunNetMinStructure]).Count + 4)] := IntToStr(PrunStoreOutputNo);
        { Config MinCycles }
        { Print neuron number of input layer }
        Cells[3, 3] := IntToStr(PrunStoreInputNo);
        { Print neuron number of hidden layers }
        if (TRealArray(PrunNetListNetConfig.Items[PrunNetMinCycles]).Count > 0) then
          for k := 0 to (TRealArray(PrunNetListNetConfig.Items[PrunNetMinCycles]).Count - 1) do
            { Print neuron number of primary configuration }
            Cells[3, (k + 4)] := FloatToStr(TRealArray(PrunNetListNetConfig.Items[PrunNetMinCycles]).Items[k]);
        { Print neuron number of output layer }
        Cells[3, (TRealArray(PrunNetListNetConfig.Items[PrunNetMinCycles]).Count + 4)] := IntToStr(PrunStoreOutputNo);
      end;
      { Enable elements }
      PrunGroupBoxConfig.Enabled := True;
      MessageBeep(0);
      MessageDlg('Optimierung beendet. Konfiguration herstellen!', mtInformation, [mbOk], 0);
    finally
      { Set status }
      BackPropSetStatus(0, 'Warte auf Eingabe');
      BackPropStatusBar.Refresh;
      { Reset WaitCursor }
      Screen.Cursor := crDefault;
    end
  else begin
    MessageBeep(0);
    MessageDlg('Optimierung nicht gestartet. Aktuelles Netz unverndert.', mtInformation, [mbOk], 0);
  end;
end;

procedure TBackPropForm.PrunFreeAllPrunLists;
var
  n: Integer;
begin
  { Free all existing lists that have stored data of previous pruning sessions }
  { Configuration of optimized nets }
  if Assigned(PrunNetListNetConfig) then begin
    with PrunNetListNetConfig do begin
      if (Count > 0) then begin
        for n := 0 to (Count - 1) do begin
          TRealArray(Items[n]).Free;
        end;
      end;
      Free;
      PrunNetListNetConfig := nil;
    end;
  end;
  { LearnCycles of optimized nets }
  if Assigned(PrunNetListLearnCycles) then begin
    with PrunNetListLearnCycles do Free;
    PrunNetListLearnCycles := nil;
  end;
end;

procedure TBackPropForm.PrunSpinEditLimitChange(Sender: TObject);
var
  HelpValueInteger: Integer;
begin
  with PrunSpinEditLimit do try
    HelpValueInteger := Value;
    { Check the given value for conflict with net component }
    if (HelpValueInteger < 1) or (HelpValueInteger > 200) then begin
      MessageBeep(0);
      MessageDlg('Wert fr Relevanzgrenze prfen (minimal 1%, maximal 200%)!', mtError, [mbOk], 0);
      Value := 50;
      SetFocus;
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Relevanzgrenze muss Integer sein!', mtError, [mbOk], 0);
      Value := 50;
      SetFocus;
    end;
  end;
end;

{ TabSheet 'Recall' following }

procedure TBackPropForm.RecallBitBtnBreakClick(Sender: TObject);
begin
  { Interrupt recall process }
  BackPropBackPropNet.Break := True;
end;

procedure TBackPropForm.RecallBitBtnNewInitClick(Sender: TObject);
begin
  with BackPropBackPropNet.RecallSet do begin
    { Delete all existing patterns }
    DeleteAll;
    { Create new PatternSet }
    PatternNo := 1;
    InputNo := BackPropBackPropNet.InputNo;
    OutputNo := BackPropBackPropNet.Items[(BackPropBackPropNet.LayerNo - 1)].NeuronNo;
    { Delete number of recall vectors in ComboBoxes }
    RecallComboBoxRecallVectInp.Clear;
    RecallComboBoxRecallVectOutp.Clear;
    RecallListBoxOutp.Clear;
    { Write new recall vector numbers into ComboBoxes and set indices }
    with RecallComboBoxRecallVectInp do begin
      Items.Add(IntToStr(Items.Count + 1));
      ItemIndex := (Items.Count - 1);
    end;
    { Disable ComboBoxRecallVectOutp }
    with RecallComboBoxRecallVectOutp do begin
      Enabled := False;
      Color := clBtnFace;
    end;
    { Set dimension in RecallEditRecallSetDimInp }
    RecallEditRecallSetDimInp.Text := IntToStr(InputNo);
    RecallPanelRecallSetDimOutp.Caption := '';
  end;
  { Setting panel showing total number of recall vectors }
  RecallPanelNrRecallVect.Caption := IntToStr(RecallComboBoxRecallVectInp.Items.Count);
  { Call procedures to set ListBoxes }
  RecallComboBoxRecallVectInpChange(Self);
  { Write file name in Edit }
  RecallEditFileRcl.Text := 'Untitled.rcl';
end;

procedure TBackPropForm.RecallBitBtnNewRecallVectClick(Sender: TObject);
begin
  with RecallComboBoxRecallVectInp do begin
    { Set ComboBox Items }
    Items.Add(IntToStr(Items.Count + 1));
    ItemIndex := (Items.Count - 1);
    {Create new recall vector at the end of the set }
    with BackPropBackPropNet do RecallSet.PatternNo := (RecallSet.PatternNo + 1);
    { Setting panel showing total number of recall vectors }
    RecallPanelNrRecallVect.Caption := IntToStr(Items.Count);
  end;
  RecallComboBoxRecallVectInpChange(Self);
end;

procedure TBackPropForm.RecallBitBtnOpenRecallSetClick(Sender: TObject);
var
  i: Integer;
begin
  with BackPropNetFileDialog do try
    DialogMode := dmOpen;
    Title := 'ffnen einer ReproduktionsSetDatei';
    { Filter on Recall }
    FilterIndex := 6;
    FileName := RecallEditFileRcl.Text;
    DefaultExt := 'rcl';
    if (Execute = True) then begin
      BackPropBackPropNet.LoadFromFile(FileName);
      { Write FileName in RecallEditFileRcl }
      RecallEditFileRcl.Text := ExtractFileName(FileName);
    end;
    { Input ComboBox }
    with RecallComboBoxRecallVectInp do begin
      Clear;
      for i := 0 to (BackPropBackPropNet.RecallSet.PatternNo - 1) do
        Items.Add(IntToStr(Items.Count + 1));
      ItemIndex := (Items.Count - 1);
    end;
    { Output ComboBox }
    with RecallComboBoxRecallVectOutp do begin
      Clear;
      Enabled := True;
      Color := clWindow;
      for i := 0 to (BackPropBackPropNet.RecallSet.PatternNo - 1) do
        Items.Add(IntToStr(Items.Count + 1));
      ItemIndex := (Items.Count - 1);
    end;
    { Set dimension in RecallEditRecallSetDimInp }
    with BackPropBackPropNet do begin
      RecallEditRecallSetDimInp.Text := IntToStr(RecallSet.InputNo);
      RecallPanelRecallSetDimOutp.Caption := IntToStr(RecallSet.OutputNo);
      { Setting panel showing total number of recall vectors }
      RecallPanelNrRecallVect.Caption := IntToStr(RecallSet.PatternNo);
    end;
    { Call procedures to set ListBoxes }
    RecallComboBoxRecallVectInpChange(Self);
    RecallComboBoxRecallVectOutpChange(Self);
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TBackPropForm.RecallBitBtnSaveRecallSetClick(Sender: TObject);
begin
  with BackPropNetFileDialog do try
    DialogMode := dmSave;
    Title := 'Speichern einer ReproduktionsSetDatei';
    { Filter on Recall }
    FilterIndex := 6;
    FileName := RecallEditFileRcl.Text;
    DefaultExt := 'rcl';
    if Execute then begin
      BackPropBackPropNet.SaveToFile(FileName);
      RecallEditFileRcl.Text := ExtractFileName(FileName);
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TBackPropForm.RecallBitBtnDelRecallVectClick(Sender: TObject);
begin
  with RecallComboBoxRecallVectInp do begin
    { Delete last recall vector of RecallSet? }
    if (Items.Count = 1) then begin
      MessageBeep(0);
      if (MessageDlg('Nur noch ein Vektor im Set. Lschen und Set neu setzen?',
        mtConfirmation, [mbYes, mbNo], 0) = mrYes) then begin
        { Create new RecallSet }
        with BackPropBackPropNet do begin
          RecallSet.DeleteAll;
          RecallSet.PatternNo := 1;
        end;
        { Delete number of recall vectors in ComboBox }
        Clear;
        { Write new recall vector number into ComboBox }
        Items.Add(IntToStr(Items.Count + 1));
        ItemIndex := (Items.Count - 1);
        { Set dimension in RecallPanelRecallSetDimInp }
        RecallEditRecallSetDimInp.Text := IntToStr(BackPropBackPropNet.RecallSet.InputNo);
      end;
    end
    else if (Items.Count >= 2) then begin
      { Delete RecallVector in net component }
      BackPropBackPropNet.RecallSet.Delete(ItemIndex);
      { Change data in ComboBox }
      Items.Delete(Items.Count - 1);
    end;
    { Setting panel showing total number of RecallVectors }
    RecallPanelNrRecallVect.Caption := IntToStr(Items.Count);
  end;
  { Set values in other boxes and elements }
  RecallComboBoxRecallVectInpChange(Self);
end;

procedure TBackPropForm.RecallBitBtnRecallSetDimInpDecClick(Sender: TObject);
begin
  with BackPropBackPropNet.RecallSet do
    if (InputNo > 1) then begin
      InputNo := (InputNo - 1);
      RecallEditRecallSetDimInp.Text := IntToStr(InputNo);
      RecallComboBoxRecallVectInpChange(Self);
    end
    else begin
      MessageBeep(0);
      MessageDlg('Dimension des Eingangs kann nicht < 1 sein!', mtWarning, [mbOk], 0);
    end;
end;

procedure TBackPropForm.RecallBitBtnRecallSetDimInpIncClick(Sender: TObject);
begin
  with BackPropBackPropNet.RecallSet do begin
    InputNo := (InputNo + 1);
    RecallEditRecallSetDimInp.Text := IntToStr(InputNo);
  end;
  RecallComboBoxRecallVectInpChange(Self);
end;

procedure TBackPropForm.RecallBitBtnStartClick(Sender: TObject);
var
  i: Integer;
begin
  try
    { Set WaitCursor }
    Screen.Cursor := crBackPropWaitCursor;
    { Set status }
    BackPropSetStatus(1, 'ParameterCheck wird durchgefhrt');
    BackPropStatusBar.Refresh;
    { Set procedures for events within net component }
    with BackPropBackPropNet do OnExit := BackPropBackPropNetRecallExit;
    { Set status }
    BackPropSetStatus(2, 'Reproduziere Werte');
    BackPropStatusBar.Refresh;
    { Start recall procedure }
    BackPropBackPropNet.Recall;
    with RecallComboBoxRecallVectOutp do begin
      { Delete number of recall vectors in ComboBox }
      Clear;
      { Write new recall vector number into ComboBox }
      for i := 0 to (BackPropBackPropNet.RecallSet.PatternNo - 1) do Items.Add(IntToStr(i + 1));
      ItemIndex := (Items.Count - 1);
      { Enable ComboBox for choosing vector to be shown in ListBox }
      Enabled := True;
      Color := clWindow;
    end;
    { Set dimension in RecallEditRecallSetDimInp }
    RecallPanelRecallSetDimOutp.Caption := IntToStr(BackPropBackPropNet.RecallSet.OutputNo);
    RecallComboBoxRecallVectOutpChange(Self);
  finally
    { Set status }
    BackPropSetStatus(0, 'Warte auf Eingabe');
    BackPropStatusBar.Refresh;
    { Reset WaitCursor }
    Screen.Cursor := crDefault;
  end;
end;

procedure TBackPropForm.RecallComboBoxRecallVectInpChange(Sender: TObject);
var
  i: Integer;
begin
  { Set listboxes }
  RecallListBoxInp.Clear;
  with RecallComboBoxRecallVectInp do
    if (Items.Count > 0) then begin
      { Setting labels showing size of input of RecallVector }
      RecallLabelListInp.Caption := 'EingangsVektor ( ' +
        IntToStr(BackPropBackPropNet.RecallSet.InputNo) + ' )';
      { Fill ListBoxes if input vectors exist }
      for i := 0 to (BackPropBackPropNet.RecallSet.InputNo - 1) do
        RecallListBoxInp.Items.Add(FloatToStr(BackPropBackPropNet.RecallSet.Items[ItemIndex].Inputs[i]));
      RecallListBoxInp.ItemIndex := 0;
    end
    else begin
      { Setting labels showing emptiness of input vectors }
      RecallLabelListInp.Caption := 'EingangsVektor';
      { Set ListBoxes to empty }
      RecallListBoxInp.ItemIndex := -1;
    end;
end;

procedure TBackPropForm.RecallComboBoxRecallVectOutpChange(Sender: TObject);
var
  i: Integer;
begin
  { Set ListBox }
  RecallListBoxOutp.Clear;
  with RecallComboBoxRecallVectOutp do
    if (Items.Count > 0) then begin
      { Setting labels showing size of output of RecallVector }
      RecallLabelListOutp.Caption := 'AusgangsVektor ( ' + IntToStr(BackPropBackPropNet.RecallSet.OutputNo) + ' )';
      { Fill ListBox if output vector exists }
      for i := 0 to (BackPropBackPropNet.RecallSet.OutputNo - 1) do
        RecallListBoxOutp.Items.Add(FloatToStr(BackPropBackPropNet.RecallSet.Items[ItemIndex].Outputs[i]));
      { Set ItemIndex to first element }
      RecallListBoxOutp.ItemIndex := 0;
    end
    else begin
      { Setting labels showing emptiness of output vector }
      RecallLabelListOutp.Caption := 'AusgangsVektor';
      { Set ListBoxes to empty }
      RecallListBoxOutp.ItemIndex := - 1;
    end;
end;

procedure TBackPropForm.RecallEditRecallSetDimInpExit(Sender: TObject);
var
  HelpValue: Integer;
begin
  with RecallEditRecallSetDimInp do try
    HelpValue := StrToInt(Text);
    if (HelpValue < 1) then begin
      MessageBeep(0);
      MessageDlg('Dimension des Eingangs kann nicht < 1 sein!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.RecallSet.InputNo);
      SetFocus;
    end
    else begin
      BackPropBackPropNet.RecallSet.InputNo := HelpValue;
      RecallComboBoxRecallVectInpChange(Self);
    end;
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Dimension des Eingangsvektors prfen!', mtError, [mbOk], 0);
      Text := IntToStr(BackPropBackPropNet.RecallSet.InputNo);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.RecallEditRecallSetDimInpKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then RecallEditRecallSetDimInpExit(Self);
end;

procedure TBackPropForm.RecallEditValInpCurrExit(Sender: TObject);
begin
  with RecallEditValInpCurr do try
    BackPropBackPropNet.RecallSet.Items[RecallComboBoxRecallVectInp.ItemIndex].
      Inputs[RecallListBoxInp.ItemIndex] := StrToFloat(Text);
    RecallComboBoxRecallVectInpChange(Self);
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Eingangsvektor prfen!', mtError, [mbOk], 0);
      Text := FloatToStr(BackPropBackPropNet.RecallSet.
        Items[RecallComboBoxRecallVectInp.ItemIndex].Inputs[RecallListBoxInp.ItemIndex]);
      SetFocus;
    end;
  end;
end;

procedure TBackPropForm.RecallEditValInpCurrKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then RecallEditValInpCurrExit(Self);
end;

procedure TBackPropForm.RecallListBoxInpExit(Sender: TObject);
begin
  with RecallEditValInpCurr do begin
    Text := RecallListBoxInp.Items[RecallListBoxInp.ItemIndex];
    SetFocus;
  end;
end;

procedure TBackPropForm.RecallListBoxInpKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then RecallListBoxInpExit(Self);
end;

end.
